/************************************************************************//**
 *
 * @file    clg_m20_m21_m22_m23_m24_m25_reg.h
 *
 * @brief   Definition of the Clock Generator(CLG) Register for S1C17M20/21/22/23/24/25.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *
 ******************************************************************************/
#ifndef CLG_REG_H__
#define CLG_REG_H__

#ifdef __cplusplus
extern "C" {
#endif

/// CLG System Clock Control Register
typedef union CLGSCLK_REG_TAG {
    unsigned short  reg;
    struct CLGSCLK_REG_BIT_TAG {
        /**
         * These bits select the SYSCLK clock source.
         * .CLKSRC[1:0]
         *  These bits select the SYSCLK clock source.
         *  When a currently stopped clock source is selected, it will automatically start oscillating or clock input.
         * .CLKDIV[1:0]
         *  These bits set the division ratio of the clock source to determine the SYSCLK frequency.
         * +---------------+----------------------------------------+
         * |               |                CLKSRC[1:0]             |
         * |               +---------+---------+---------+----------+
         * |               |   0x0   |   0x1   |   0x2   |   0x3    |
         * |  CLKDIV[1:0]  +---------+---------+---------+----------+
         * |               | IOSCCLK | OSC1CLK | OSC3CLK | EXOSCCLK |
         * +---------------+---------+---------+---------+----------+
         * |      0x3      |   1/8   |Reserved |   1/8   | Reserved |
         * +---------------+---------+         +---------+          +
         * |      0x2      |   1/4   |         |   1/4   |          |
         * +---------------+---------+---------+---------+          +
         * |      0x1      |   1/2   |  1/2    |   1/2   |          |
         * +---------------+---------+---------+---------+----------+
         * |      0x0      |   1/1   |  1/1    |   1/1   |   1/1    |
         * +---------------+---------+---------+--------------------+
         */
        unsigned short      CLKSRC      : 2;
        unsigned short      reserved_1  : 2;    ///< Reserved.
        /**
         * These bits set the division ratio of the clock source to determine the SYSCLK frequency.
         * .CLKSRC[1:0]
         *  These bits select the SYSCLK clock source.
         *  When a currently stopped clock source is selected, it will automatically start oscillating or clock input.
         * .CLKDIV[1:0]
         *  These bits set the division ratio of the clock source to determine the SYSCLK frequency.
         * +---------------+----------------------------------------+
         * |               |                CLKSRC[1:0]             |
         * |               +---------+---------+---------+----------+
         * |               |   0x0   |   0x1   |   0x2   |   0x3    |
         * |  CLKDIV[1:0]  +---------+---------+---------+----------+
         * |               | IOSCCLK | OSC1CLK | OSC3CLK | EXOSCCLK |
         * +---------------+---------+---------+---------+----------+
         * |      0x3      |   1/8   |Reserved |   1/8   | Reserved |
         * +---------------+---------+         +---------+          +
         * |      0x2      |   1/4   |         |   1/4   |          |
         * +---------------+---------+---------+---------+          +
         * |      0x1      |   1/2   |  1/2    |   1/2   |          |
         * +---------------+---------+---------+---------+----------+
         * |      0x0      |   1/1   |  1/1    |   1/1   |   1/1    |
         * +---------------+---------+---------+--------------------+
         */
        unsigned short      CLKDIV      : 2;
        unsigned short      reserved_2  : 2;    ///< Reserved.
        /**
         * These bits select the SYSCLK clock source for resetting the CLGSCLK.CLKSRC[1:0] bits at wake-up.
         * .WUPSRC[1:0]
         *  These bits select the SYSCLK clock source for resetting the CLGSCLK.CLKSRC[1:0] bits at wake-up.
         *  When a currently stopped clock source is selected, it will automatically start oscillating or clock input
         *  at wake-up. However, this setting is ineffective when the CLGSCLK.WUPMD bit = 0.
         *  When a currently stopped clock source is selected, it will automatically start oscillating or clock input.
         * .WUPDIV[1:0]
         *  These bits select the SYSCLK division ratio for resetting the CLGSCLK.CLKDIV[1:0] bits at wake-up.
         *  This setting is ineffective when the CLGSCLK.WUPMD bit = 0.
         * +---------------+----------------------------------------+
         * |               |                WUPSRC[1:0]             |
         * |               +---------+---------+---------+----------+
         * |               |   0x0   |   0x1   |   0x2   |   0x3    |
         * |  WUPDIV[1:0]  +---------+---------+---------+----------+
         * |               | IOSCCLK | OSC1CLK | OSC3CLK | EXOSCCLK |
         * +---------------+---------+---------+---------+----------+
         * |      0x3      |   1/8   |Reserved |   1/8   | Reserved |
         * +---------------+---------+         +---------+          +
         * |      0x2      |   1/4   |         |   1/4   |          |
         * +---------------+---------+---------+---------+          +
         * |      0x1      |   1/2   |   1/2   |   1/2   |          |
         * +---------------+---------+---------+---------+----------+
         * |      0x0      |   1/1   |   1/1   |   1/1   |   1/1    |
         * +---------------+---------+---------+--------------------+
         */
        unsigned short      WUPSRC      : 2;
        unsigned short      reserved_3  : 2;    ///< Reserved.
        /**
         * These bits select the SYSCLK division ratio for resetting the CLGSCLK.CLKDIV[1:0] bits at wake-up.
         * .WUPSRC[1:0]
         *  These bits select the SYSCLK clock source for resetting the CLGSCLK.CLKSRC[1:0] bits at wake-up.
         *  When a currently stopped clock source is selected, it will automatically start oscillating or clock input
         *  at wake-up. However, this setting is ineffective when the CLGSCLK.WUPMD bit = 0.
         *  When a currently stopped clock source is selected, it will automatically start oscillating or clock input.
         * .WUPDIV[1:0]
         *  These bits select the SYSCLK division ratio for resetting the CLGSCLK.CLKDIV[1:0] bits at wake-up.
         *  This setting is ineffective when the CLGSCLK.WUPMD bit = 0.
         * +---------------+----------------------------------------+
         * |               |                WUPSRC[1:0]             |
         * |               +---------+---------+---------+----------+
         * |               |   0x0   |   0x1   |   0x2   |   0x3    |
         * |  WUPDIV[1:0]  +---------+---------+---------+----------+
         * |               | IOSCCLK | OSC1CLK | OSC3CLK | EXOSCCLK |
         * +---------------+---------+---------+---------+----------+
         * |      0x3      |   1/8   |Reserved |   1/8   | Reserved |
         * +---------------+---------+         +---------+          +
         * |      0x2      |   1/4   |         |   1/4   |          |
         * +---------------+---------+---------+---------+          +
         * |      0x1      |   1/2   |   1/2   |   1/2   |          |
         * +---------------+---------+---------+---------+----------+
         * |      0x0      |   1/1   |   1/1   |   1/1   |   1/1    |
         * +---------------+---------+---------+--------------------+
         */
        unsigned short      WUPDIV      : 2;
        unsigned short      reserved_4  : 1;    ///< Reserved.
        /**
         * This bit enables the SYSCLK switching function at wake-up.
         * .WUPMD
         *  This bit enables the SYSCLK switching function at wake-up.
         *    1 (R/WP): Enable
         *    0 (R/WP): Disable
         */
        unsigned short      WUPMD       : 1;
    } bit;
    unsigned char   byte_reg[2];
} CLGSCLK_REG;

/// CLG Oscillation Control Register.
typedef union CLGOSC_REG_TAG {
    unsigned short  reg;
    struct CLGOSC_REG_BIT_TAG {
        /**
         * This bit control the clock source(IOSC) operation.
         *  These bits control the clock source operation.
         *    1 (R/W): Start oscillating or clock input
         *    0 (R/W): Stop oscillating or clock input
         * 
         * .IOSCEN: IOSC oscillator circuit
         */
        unsigned short      IOSCEN      : 1;
        /**
         * This bit control the clock source(OSC1) operation.
         *  These bits control the clock source operation.
         *    1 (R/W): Start oscillating or clock input
         *    0 (R/W): Stop oscillating or clock input
         * 
         * .OSC1EN: OSC1 oscillator circuit
         */
        unsigned short      OSC1EN      : 1;
        /**
         * This bit control the clock source(OSC3) operation.
         *  These bits control the clock source operation.
         *    1 (R/W): Start oscillating or clock input
         *    0 (R/W): Stop oscillating or clock input
         * 
         * .OSC3EN: OSC3 oscillator circuit
         */
        unsigned short      OSC3EN      : 1;
        /**
         * This bit control the clock source(EXOSC) operation.
         *  These bits control the clock source operation.
         *    1 (R/W): Start oscillating or clock input
         *    0 (R/W): Stop oscillating or clock input
         * 
         * .EXOSCEN: EXOSC clock input
         */
        unsigned short      EXOSCEN     : 1;
        unsigned short      reserved_1  : 4;    ///< Reserved.
        /**
         * This bit control the clock source(IOSC) operations in SLEEP mode.
         *  These bits control the clock source operations in SLEEP mode.
         *    1 (R/W): Stop clock source in SLEEP mode
         *    0 (R/W): Continue operation state before SLEEP
         * 
         * .IOSCSLPC: IOSC oscillator circuit
         */
        unsigned short      IOSCSLPC    : 1;
        /**
         * This bit control the clock source(OSC1) operations in SLEEP mode.
         *  These bits control the clock source operations in SLEEP mode.
         *    1 (R/W): Stop clock source in SLEEP mode
         *    0 (R/W): Continue operation state before SLEEP
         * 
         * .OSC1SLPC: OSC1 oscillator circuit
         */
        unsigned short      OSC1SLPC    : 1;
        /**
         * This bit control the clock source(OSC3) operations in SLEEP mode.
         *  These bits control the clock source operations in SLEEP mode.
         *    1 (R/W): Stop clock source in SLEEP mode
         *    0 (R/W): Continue operation state before SLEEP
         * 
         * .OSC3SLPC: OSC3 oscillator circuit
         */
        unsigned short      OSC3SLPC    : 1;
        /**
         * This bit control the clock source(EXOSC) operations in SLEEP mode.
         *  These bits control the clock source operations in SLEEP mode.
         *    1 (R/W): Stop clock source in SLEEP mode
         *    0 (R/W): Continue operation state before SLEEP
         * 
         * .EXOSCSLPC: EXOSC clock input
         */
        unsigned short      EXOSCSLPC   : 1;
        unsigned short      reserved_2  : 4;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} CLGOSC_REG;

/// CLG OSC1 Control Register.
typedef union CLGOSC1_REG_TAG {
    unsigned short  reg;
    struct CLGOSC1_REG_BIT_TAG {
        /**
         * These bits set the oscillation stabilization waiting time for the OSC1 oscillator circuit.
         * .OSC1WT[1:0]
         *  These bits set the oscillation stabilization waiting time for the OSC1 oscillator circuit.
         * +--------------+--------------------------------------+
         * |  OSC1WT[1:0] |Oscillation stabilization waiting time|
         * +--------------+--------------------------------------+
         * |     0x3      |          65,536 clocks               |
         * +--------------+--------------------------------------+
         * |     0x2      |          16,384 clocks               |
         * +--------------+--------------------------------------+
         * |     0x1      |           4,096 clocks               |
         * +--------------+--------------------------------------+
         * |     0x0      |             Reserved                 |
         * +--------------+--------------------------------------+
         */
        unsigned short      OSC1WT      : 2;
        unsigned short      reserved_1  : 2;    ///< Reserved.
        /**
         * These bits set the oscillation inverter gain applied at normal operation of the OSC1 oscillator circuit.
         * .INV1N[1:0]
         *  These bits set the oscillation inverter gain applied at normal operation of the OSC1 oscillator circuit.
         * +------------+---------------+
         * | INV1N[1:0] | Inverter gain |
         * +------------+---------------+
         * |    0x3     |   Max.        |
         * +------------+               +
         * |    0x2     |               |
         * +------------+               +
         * |    0x1     |               |
         * +------------+               +
         * |    0x0     |   Min.        |
         * +------------+---------------+
         */
        unsigned short      INV1N       : 2;
        /**
         * These bits set the oscillation inverter gain that will be applied at boost startup of the OSC1 oscillator circuit.
         * .INV1B[1:0]
         *  These bits set the oscillation inverter gain that will be applied at boost startup of the OSC1 oscillator circuit.
         * +------------+---------------+
         * | INV1B[1:0] | Inverter gain |
         * +------------+---------------+
         * |    0x3     |   Max.        |
         * +------------+               +
         * |    0x2     |               |
         * +------------+               +
         * |    0x1     |               |
         * +------------+               +
         * |    0x0     |   Min.        |
         * +------------+---------------+
         */
        unsigned short      INV1B       : 2;
        /**
         * These bits set the internal gate capacitance in the OSC1 oscillator circuit.
         * .CGI1[2:0]
         *  These bits set the internal gate capacitance in the OSC1 oscillator circuit.
         * +-----------+---------------+
         * | CGI1[2:0] | Inverter gain |
         * +-----------+---------------+
         * |    0x7    |   Max.        |
         * +-----------+               +
         * |    0x6    |               |
         * +-----------+               +
         * |    0x5    |               |
         * +-----------+               +
         * |    0x4    |               |
         * +-----------+               +
         * |    0x3    |               |
         * +-----------+               +
         * |    0x2    |               |
         * +-----------+               +
         * |    0x1    |               |
         * +-----------+               +
         * |    0x0    |   Min.        |
         * +-----------+---------------+
         */
        unsigned short      CGI1        : 3;
        /**
         * This This bit selects an oscillator type of the OSC1 oscillator circuit.
         * .OSC1SELCR
         *  This bit enables the oscillation startup control circuit in the OSC1 oscillator circuit.
         *    1 (R/WP): Internal oscillator
         *    0 (R/WP): Crystal oscillator
         */
        unsigned short      OSC1SELCR   : 1;
        /**
         * This bit enables the oscillation startup control circuit in the OSC1 oscillator circuit.
         * .OSC1BUP
         *  This bit enables the oscillation startup control circuit in the OSC1 oscillator circuit.
         *    1 (R/WP): Enable (Activate booster operation at startup.)
         *    0 (R/WP): Disable
         */
        unsigned short      OSC1BUP     : 1;
        /**
         * This bit controls the oscillation stop detector in the OSC1 oscillator circuit.
         * .OSDEN
         *  This bit controls the oscillation stop detector in the OSC1 oscillator circuit.
         *    1 (R/WP): OSC1 oscillation stop detector on
         *    0 (R/WP): OSC1 oscillation stop detector off
         */
        unsigned short      OSDEN       : 1;
        /**
         * This bit enables the OSC1 oscillator circuit restart function by the oscillation stop detector when OSC1 oscillation stop is detected.
         * .OSDRB
         *  This bit enables the OSC1 oscillator circuit restart function by the oscillation stop detector when OSC1 oscillation stop is detected.
         *    1 (R/WP): Enable (Restart the OSC1 oscillator circuit when oscillation stop is detected.)
         *    0 (R/WP): Disable
         */
        unsigned short      OSDRB       : 1;
        unsigned short      reserved_2  : 1;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} CLGOSC1_REG;

/// CLG OSC3 Control Register.
typedef union CLGOSC3_REG_TAG {
    unsigned short  reg;
    struct CLGOSC3_REG_BIT_TAG {
        /**
         * These bits set the oscillation stabilization waiting time for the OSC3 oscillator circuit.
         * .OSC3WT[2:0]
         *  These bits set the oscillation stabilization waiting time for the OSC3 oscillator circuit.
         * +-------------+---------------+
         * |             | Oscillation   |
         * | OSC3WT[2:0] | stabilization |
         * |             | waiting time  |
         * +-------------+---------------+
         * |     0x7     | 65,536 clocks |
         * +-------------+---------------+
         * |     0x6     | 16,384 clocks |
         * +-------------+---------------+
         * |     0x5     |  4,096 clocks |
         * +-------------+---------------+
         * |     0x4     |  1,024 clocks |
         * +-------------+---------------+
         * |     0x3     |    256 clocks |
         * +-------------+---------------+
         * |     0x2     |     64 clocks |
         * +-------------+---------------+
         * |     0x1     |     16 clocks |
         * +-------------+---------------+
         * |     0x0     |      4 clocks |
         * +-------------+---------------+ 
         */
        unsigned short      OSC3WT      : 3;
        /**
         * This bit controls the OSC3 internal oscillator auto-trimming function.
         * .OSC3STM
         *  This bit controls the OSC3 internal oscillator auto-trimming function.
         *    1 (WP): Start trimming
         *    0 (WP): Stop trimming
         *    1 (R): Trimming is executing.
         *    0 (R): Trimming has finished. (Trimming operation inactivated.)
         */
        unsigned short      OSC3STM     : 1;
        /**
         * These bits set the oscillation inverter gain of the OSC3 oscillator circuit.
         * .OSC3INV[1:0]
         *  These bits set the oscillation inverter gain of the OSC3 crystal/ceramic oscillator circuit.
         * +--------------+---------------+
         * | OSC3INV[1:0] | Inverter gain |
         * +--------------+---------------+
         * |      0x3     |     Max.      |
         * +--------------+               +
         * |      0x2     |               |
         * +--------------+               +
         * |      0x1     |               |
         * +--------------+               +
         * |      0x0     |     Min.      |
         * +--------------+---------------+
         */
        unsigned short      OSC3INV     : 2;
        unsigned short      reserved_1  : 3;    ///< Reserved.
        /**
         * This bit selects an oscillator type of the OSC3 oscillator circuit.
         * .OSC3MD
         *  This bit selects an oscillator type of the OSC3 oscillator circuit.
         *    1 (R/WP): Crystal/ceramic oscillator
         *    0 (R/WP): Internal oscillator
         */
        unsigned short      OSC3MD      : 1;
        /**
         * These bits set the oscillation frequency of the OSC3 internal oscillator circuit.
         * .OSC3FQ[1:0]
         *  These bits set the oscillation frequency of the OSC3 internal oscillator circuit.
         * +--------------+----------------------------+
         * | OSC3FQ[1:0]  | OSC3 oscillation frequency |
         * +--------------+----------------------------+
         * |      0x3     |          Reserved          |
         * +--------------+----------------------------+
         * |      0x2     |           20 MHz           |
         * +--------------+----------------------------+
         * |      0x1     |           16 MHz           |
         * +--------------+----------------------------+
         * |      0x0     |           12 MHz           |
         * +--------------+----------------------------+
         */
        unsigned short      OSC3FQ      : 2;
        unsigned short      reserved_2  : 4;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} CLGOSC3_REG;

/// CLG Interrupt Flag Register.
typedef union CLGINTF_REG_TAG {
    unsigned short  reg;
    struct CLGINTF_REG_BIT_TAG {
        /**
         * This bit indicate the oscillation stabilization waiting completion interrupt cause occurrence status in each clock source(IOSC).
         *  This bit indicate the CLG interrupt cause occurrence statuses.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         * .IOSCSTAIF: IOSC oscillation stabilization waiting completion interrupt
         */
        unsigned short      IOSCSTAIF   : 1;
        /**
         * This bit indicate the oscillation stabilization waiting completion interrupt cause occurrence status in each clock source(OSC1).
         *  This bit indicate the CLG interrupt cause occurrence statuses.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         * .OSC1STAIF: OSC1 oscillation stabilization waiting completion interrupt
         */
        unsigned short      OSC1STAIF   : 1;
        /**
         * This bit indicate the oscillation stabilization waiting completion interrupt cause occurrence status in each clock source(OSC3).
         *  This bit indicate the CLG interrupt cause occurrence statuses.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         * .OSC3STAIF: OSC3 oscillation stabilization waiting completion interrupt
         */
        unsigned short      OSC3STAIF   : 1;
        unsigned short      reserved_1  : 1;    ///< Reserved.
        /**
         * This bit indicate the OSC3 oscillation auto-trimming completion interrupt cause occurrence statuses.
         *  These bits indicate the CLG interrupt cause occurrence statuses.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         * .OSC3TEDIF: IOSC oscillation auto-trimming completion interrupt
         */
        unsigned short      OSC3TEDIF   : 1;
        /**
         * This bit indicate the OSC1 oscillation stop interrupt cause occurrence statuses.
         *  These bits indicate the CLG interrupt cause occurrence statuses.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         * .OSC1STPIF: OSC1 oscillation auto-trimming completion interrupt
         */
        unsigned short      OSC1STPIF   : 1;
        unsigned short      reserved_2  :10;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} CLGINTF_REG;

/// CLG Interrupt Enable Register.
typedef union CLGINTE_REG_TAG {
    unsigned short  reg;
    struct CLGINTE_REG_BIT_TAG {
        /**
         * This bit enable the oscillation stabilization waiting completion interrupt of each clock source(IOSC).
         *  This bit enable the CLG interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         * .IOSCSTAIE: IOSC oscillation stabilization waiting completion interrupt
         */
        unsigned short      IOSCSTAIE   : 1;
        /**
         * This bit enable the oscillation stabilization waiting completion interrupt of each clock source(OSC1).
         *  This bit enable the CLG interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         * .OSC1STAIE: OSC1 oscillation stabilization waiting completion interrupt
         */
        unsigned short      OSC1STAIE   : 1;
        /**
         * This bit enable the oscillation stabilization waiting completion interrupt of each clock source(OSC3).
         *  This bit enable the CLG interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         * .OSC3STAIE: OSC3 oscillation stabilization waiting completion interrupt
         */
        unsigned short      OSC3STAIE   : 1;
        unsigned short      reserved_1  : 1;    ///< Reserved.
        /**
         * This bit enable the OSC3 oscillation auto-trimming completion interrupt.
         *  This bit enable the CLG interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         * .OSC3TEDIE: OSC3 oscillation auto-trimming completion interrupt
         */
        unsigned short      OSC3TEDIE   : 1;
        /**
         * This bit enable the OSC1 oscillation stop.
         *  These bits enable the CLG interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         * .OSC1STPIE: OSC1 oscillation stop interrupt
         */
        unsigned short      OSC1STPIE   : 1;
        unsigned short      reserved_2  :10;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} CLGINTE_REG;

/// CLG FOUT Control Register.
typedef union CLGFOUT_REG_TAG {
    unsigned short  reg;
    struct CLGFOUT_REG_BIT_TAG {
        /**
         * This bit controls the FOUT clock external output.
         * .FOUTEN
         * This bit controls the FOUT clock external output.
         *    1 (R/W): Enable external output
         *    0 (R/W): Disable external output
         */
        unsigned short      FOUTEN      : 1;
        unsigned short      reserved_1  : 1;    ///< Reserved.
        /**
         * These bits select the FOUT clock source.
         * .FOUTSRC[1:0]
         *  These bits select the FOUT clock source.
         * .FOUTDIV[2:0]
         *  These bits set the FOUT clock division ratio.
         * +---------------+-----------------------------------------+
         * |               |                FOUTSRC[1:0]             |
         * |               +---------+----------+---------+----------+
         * |               |   0x0   |   0x1    |   0x2   |   0x3    |
         * | FOUTDIV[2:0]  +---------+----------+---------+----------+
         * |               | IOSCCLK |  OSC1CLK | OSC3CLK |  SYSCLK  |
         * +---------------+---------+----------+---------+----------+
         * |      0x7      |  1/128  | 1/32,768 |  1/128  | Reserved |
         * +---------------+---------+----------+---------+          +
         * |      0x6      |   1/64  |  1/4,096 |  1/64   |          |
         * +---------------+---------+----------+---------+          +
         * |      0x5      |   1/32  |  1/1,024 |  1/32   |          |
         * +---------------+---------+----------+---------+          +
         * |      0x4      |   1/16  |  1/256   |  1/16   |          |
         * +---------------+---------+----------+---------+          +
         * |      0x3      |   1/8   |   1/8    |  1/8    |          |
         * +---------------+---------+----------+---------+          +
         * |      0x2      |   1/4   |   1/4    |  1/4    |          |
         * +---------------+---------+----------+---------+          +
         * |      0x1      |   1/2   |   1/2    |  1/2    |          |
         * +---------------+---------+----------+---------+----------+
         * |      0x0      |   1/1   |   1/1    |  1/1    |   1/1    |
         * +---------------+---------+----------+--------------------+
         */
        unsigned short      FOUTSRC     : 2;
        /**
         * These bits set the FOUT clock division ratio.
         * .FOUTSRC[1:0]
         *  These bits select the FOUT clock source.
         * .FOUTDIV[2:0]
         *  These bits set the FOUT clock division ratio.
         * +---------------+-----------------------------------------+
         * |               |                FOUTSRC[1:0]             |
         * |               +---------+----------+---------+----------+
         * |               |   0x0   |   0x1    |   0x2   |   0x3    |
         * | FOUTDIV[2:0]  +---------+----------+---------+----------+
         * |               | IOSCCLK |  OSC1CLK | OSC3CLK |  SYSCLK  |
         * +---------------+---------+----------+---------+----------+
         * |      0x7      |  1/128  | 1/32,768 |  1/128  | Reserved |
         * +---------------+---------+----------+---------+          +
         * |      0x6      |   1/64  |  1/4,096 |  1/64   |          |
         * +---------------+---------+----------+---------+          +
         * |      0x5      |   1/32  |  1/1,024 |  1/32   |          |
         * +---------------+---------+----------+---------+          +
         * |      0x4      |   1/16  |  1/256   |  1/16   |          |
         * +---------------+---------+----------+---------+          +
         * |      0x3      |   1/8   |   1/8    |  1/8    |          |
         * +---------------+---------+----------+---------+          +
         * |      0x2      |   1/4   |   1/4    |  1/4    |          |
         * +---------------+---------+----------+---------+          +
         * |      0x1      |   1/2   |   1/2    |  1/2    |          |
         * +---------------+---------+----------+---------+----------+
         * |      0x0      |   1/1   |   1/1    |  1/1    |   1/1    |
         * +---------------+---------+----------+--------------------+
         */
        unsigned short      FOUTDIV     : 3;
        unsigned short      reserved_2  : 9;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} CLGFOUT_REG;

#ifdef __cplusplus
}
#endif

#endif // CLG_REG_H__
