/************************************************************************//**
 *
 * @file    c17_clg.h
 *
 * @brief   This is clg(osc) driver header file.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Oct. 2017    1.10
 *              - Modify header file.
 *          - 01 Jun. 2018    1.20
 *              - Modify header file.
 *
 ***************************************************************************/
#ifndef C17_CLG_H__
#define C17_CLG_H__

#include <c17_mcu_select.h>

/* --- definition --- */

#if defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_M10)
enum
{
	C17_CLG_OSC3_4MHZ,
	C17_CLG_OSC3_8MHZ,
	C17_CLG_OSC3_12MHZ,
	C17_CLG_OSC3_16MHZ
};
#endif

#if defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_W15)
enum
{
	C17_CLG_OSC3_500KHZ,
	C17_CLG_OSC3_1MHZ,
	C17_CLG_OSC3_2MHZ,
	C17_CLG_OSC3_4MHZ
};
#endif

#if defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W18)
enum
{
	C17_CLG_OSC3_500KHZ,
	C17_CLG_OSC3_1MHZ,
	C17_CLG_OSC3_2MHZ,
	C17_CLG_OSC3_4MHZ,
	C17_CLG_OSC3_384KHZ,
	C17_CLG_OSC3_250KHZ
};
#endif

#if 	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25)
enum
{
	C17_CLG_OSC3_12MHZ,
	C17_CLG_OSC3_16MHZ,
	C17_CLG_OSC3_20MHZ
};
#endif

#if defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34)
enum
{
	C17_CLG_OSC3_12MHZ,
	C17_CLG_OSC3_16MHZ
};
#endif

#if defined(C17_MCUSEL_589)
enum
{
	C17_CLG_IOSC_4MHZ,
	C17_CLG_IOSC_8MHZ,
	C17_CLG_IOSC_12MHZ,
	C17_CLG_IOSC_16MHZ
};
#endif

#if defined(C17_MCUSEL_M40)
enum
{
	C17_CLG_OSC3_16MHZ
};
#endif

#if defined(C17_MCUSEL_F63)
enum
{
	C17_CLG_OSC3_500KHZ,
	C17_CLG_OSC3_1MHZ,
	C17_CLG_OSC3_2MHZ,
	C17_CLG_OSC3_4MHZ,
	C17_CLG_OSC3_8MHZ,
	C17_CLG_OSC3_12MHZ,
	C17_CLG_OSC3_16MHZ
};
#endif

#if defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_M10) || \
	defined(C17_MCUSEL_M40) || \
	defined(C17_MCUSEL_F63)
enum
{
	C17_CLG_OSC1_CRYSTAL,
	C17_CLG_OSC1_INTERNAL
};
#endif

enum
{
	C17_CLG_FOUT_IOSC,				///< FOUT Source = IOSC
	C17_CLG_FOUT_OSC1,				///< FOUT  Source = OSC1
	C17_CLG_FOUT_OSC3,				///< FOUT  Source = OSC3
	C17_CLG_FOUT_SYS				///< FOUT  Source = SYSTEM
};

enum
{
	C17_CLG_FOUT_CLKDIV_1,			///< FOUT Division = 1/1
	C17_CLG_FOUT_CLKDIV_2,			///< FOUT Division = 1/2
	C17_CLG_FOUT_CLKDIV_4,			///< FOUT Division = 1/4
	C17_CLG_FOUT_CLKDIV_8,			///< FOUT Division = 1/8
	C17_CLG_FOUT_CLKDIV_16,			///< FOUT Division = 1/16
	C17_CLG_FOUT_CLKDIV_32,			///< FOUT Division = 1/32
	C17_CLG_FOUT_CLKDIV_64,			///< FOUT Division = 1/64
	C17_CLG_FOUT_CLKDIV_128			///< FOUT Division = 1/128

};

enum
{
	C17_CLG_CONTINUE,			///< Continue operation state before SLEEP
	C17_CLG_STOP				///< Stop clock source in SLEEP mode
};

/* --- external declaration --- */
extern void c17intClg(void)	__attribute__ ((interrupt_handler));

#ifdef CLG_OSC3_ENABLE
#if defined(C17_MCUSEL_589)
#else
extern void c17startClgIntOSC3(unsigned short clgFreq);
#endif
extern void c17startClgExtOSC3(void);
extern void c17stopClgOSC3(void);
extern int c17switchClgSystemClockOSC3(void);
extern void c17setOsc3OpeInSleep(unsigned short clgSlpc);
#endif
#ifdef CLG_OSC1_ENABLE
extern void c17startClgOSC1(void);
extern void c17stopClgOSC1(void);
extern int c17switchClgSystemClockOSC1(void);
extern void c17setOsc1OpeInSleep(unsigned short clgSlpc);
extern int c17ClgAutoTrimming(void);
#if defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_M10) || \
	defined(C17_MCUSEL_M40) || \
	defined(C17_MCUSEL_F63)

extern void c17selOscTypOSC1(unsigned short selOSC1);
#endif
#endif
#ifdef CLG_IOSC_ENABLE
#if defined(C17_MCUSEL_589)
extern void c17startClgIOSC(unsigned short clgFreq);
#else
extern void c17startClgIOSC(void);
#endif
extern void c17stopClgIOSC(void);
extern int c17switchClgSystemClockIOSC(void);

extern void c17setIoscOpeInSleep(unsigned short clgSlpc);
#endif

extern void c17enableFout(unsigned short clgClkSrc, unsigned short clgClkDiv);
extern void c17disableFout(void);

#endif	// C17_CLG_H__

