/************************************************************************//**
 *
 * @file    c17_rfc.h
 *
 * @brief   This is RFC driver header file.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *
 ***************************************************************************/
#ifndef C17_RFC_H__
#define C17_RFC_H__

#include <c17_mcu_select.h>

/* --- definition --- */
enum
{
	C17_RFC_CLKSRC_IOSC,				///< Clock Source = IOSC
	C17_RFC_CLKSRC_OSC1,				///< Clock Source = OSC1
	C17_RFC_CLKSRC_OSC3,				///< Clock Source = OSC3
	C17_RFC_CLKSRC_EXT					///< Clock Source = External OSC
};

enum
{
	C17_RFC_CLKDIV_1,					///< Clock Division = 1/1
	C17_RFC_CLKDIV_2,					///< Clock Division = 1/2
	C17_RFC_CLKDIV_4,					///< Clock Division = 1/4
	C17_RFC_CLKDIV_8					///< Clock Division = 1/8
};

enum
{
	RFC_CH_0,							///< RFC CH.0
	RFC_CH_1,							///< RFC CH.1
};

// Converting Operation Status.
#define RFC_STS_CONV_SEN_A			( 0x0001 )			///< Status : Sensor A oscillation completion interrupt.
#define RFC_STS_CONV_SEN_B			( 0x0002 )			///< Status : Sensor B oscillation completion interrupt.
#define RFC_STS_CONV_ERR_OVTC		( 0x8001 )			///< Status : Time base counter overflow error.
#define RFC_STS_CONV_ERR_OVMC		( 0x8002 )			///< Status : Measurement counter overflow error.

/* --- external declaration --- */
extern void c17intRfcCh0(void)	__attribute__ ((interrupt_handler));
extern void c17intRfcCh1(void)	__attribute__ ((interrupt_handler));
extern void c17intRfcCh2(void)	__attribute__ ((interrupt_handler));
extern void c17intRfcCh3(void)	__attribute__ ((interrupt_handler));

extern void c17initRfc(void);
extern void c17setRfc(unsigned short rfcChannel, unsigned short rfcClkSrc, unsigned short rfcClkDiv, unsigned short rfcCtl);
extern void c17startRfc(unsigned short rfcChannel);
extern void c17stopRfc(unsigned short rfcChannel);
extern void c17setRfcMeasurementCounter(unsigned short rfcChannel, unsigned long count);
extern unsigned long c17getRfcMeasurementCounter(unsigned short rfcChannel);
extern void c17setRfcTimeBaseCounter(unsigned short rfcChannel, unsigned long count);
extern unsigned long c17getRfcTimeBaseCounter(unsigned short rfcChannel);
extern int c17runRfcConvertingOperationSenA(unsigned short rfcChannel, unsigned long startCount, unsigned long* sensorCount);
extern int c17runRfcConvertingOperationSenB(unsigned short rfcChannel, unsigned long startCount, unsigned long* sensorCount);
extern void c17enableIntRfc(unsigned short rfcChannel);
extern void c17disableIntRfc(unsigned short rfcChannel);

#endif /* C17_RFC_H__ */

