/************************************************************************//**
 *
 * @file    c17_spia.c
 *
 * @brief   This is SPIA driver program file.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Oct. 2017    1.10
 *              - Change source code.
 *          - 01 Jun. 2018    1.20
 *              - Change source code.
 *
 ***************************************************************************/
#include <stdlib.h>
#include "crt0.h"
#include "c17_misc.h"
#include "c17_spia.h"
#include "c17_init_config.h"

#if defined(C17_PE_SPIA_MASTER) || defined(C17_PE_SPIA_SLAVE)

/* --- definition --- */


/* --- macro definition --- */
#if defined(C17_MCUSEL_W16)
#define CH_MAX_COUNT	3
#endif
#if defined(C17_MCUSEL_M01) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25) || \
	defined(C17_MCUSEL_589) || \
	defined(C17_MCUSEL_M40)
#define CH_MAX_COUNT	2
#endif
#if defined(C17_MCUSEL_W22) || \
	defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_W15) || \
	defined(C17_MCUSEL_M10) || \
	defined(C17_MCUSEL_F63)
#define CH_MAX_COUNT	1
#endif

/* --- static const variable --- */
#if defined(C17_MCUSEL_W16)
volatile SPINMOD_REG* const spinmod_table[CH_MAX_COUNT] = {(SPINMOD_REG *)&SPI0MOD,(SPINMOD_REG *)&SPI1MOD,(SPINMOD_REG *)&SPI2MOD};
volatile SPINCTL_REG* const spinctl_table[CH_MAX_COUNT] = {(SPINCTL_REG *)&SPI0CTL,(SPINCTL_REG *)&SPI1CTL,(SPINCTL_REG *)&SPI2CTL};
volatile SPINTXD_REG* const spintxd_table[CH_MAX_COUNT] = {(SPINTXD_REG *)&SPI0TXD,(SPINTXD_REG *)&SPI1TXD,(SPINTXD_REG *)&SPI2TXD};
volatile SPINRXD_REG* const spinrxd_table[CH_MAX_COUNT] = {(SPINRXD_REG *)&SPI0RXD,(SPINRXD_REG *)&SPI1RXD,(SPINRXD_REG *)&SPI2RXD};
volatile SPININTF_REG* const spinintf_table[CH_MAX_COUNT] = {(SPININTF_REG *)&SPI0INTF,(SPININTF_REG *)&SPI1INTF,(SPININTF_REG *)&SPI2INTF};
volatile SPININTE_REG* const spininte_table[CH_MAX_COUNT] = {(SPININTE_REG *)&SPI0INTE,(SPININTE_REG *)&SPI1INTE,(SPININTE_REG *)&SPI2INTE};
#endif
#if defined(C17_MCUSEL_M01) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25) || \
	defined(C17_MCUSEL_589) || \
	defined(C17_MCUSEL_M40)
volatile SPINMOD_REG* const spinmod_table[CH_MAX_COUNT] = {(SPINMOD_REG *)&SPI0MOD,(SPINMOD_REG *)&SPI1MOD};
volatile SPINCTL_REG* const spinctl_table[CH_MAX_COUNT] = {(SPINCTL_REG *)&SPI0CTL,(SPINCTL_REG *)&SPI1CTL};
volatile SPINTXD_REG* const spintxd_table[CH_MAX_COUNT] = {(SPINTXD_REG *)&SPI0TXD,(SPINTXD_REG *)&SPI1TXD};
volatile SPINRXD_REG* const spinrxd_table[CH_MAX_COUNT] = {(SPINRXD_REG *)&SPI0RXD,(SPINRXD_REG *)&SPI1RXD};
volatile SPININTF_REG* const spinintf_table[CH_MAX_COUNT] = {(SPININTF_REG *)&SPI0INTF,(SPININTF_REG *)&SPI1INTF};
volatile SPININTE_REG* const spininte_table[CH_MAX_COUNT] = {(SPININTE_REG *)&SPI0INTE,(SPININTE_REG *)&SPI1INTE};
#endif
#if defined(C17_MCUSEL_W22) || \
	defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_W15) || \
	defined(C17_MCUSEL_M10) || \
	defined(C17_MCUSEL_F63)
volatile SPINMOD_REG* const spinmod_table[CH_MAX_COUNT] = {(SPINMOD_REG *)&SPI0MOD};
volatile SPINCTL_REG* const spinctl_table[CH_MAX_COUNT] = {(SPINCTL_REG *)&SPI0CTL};
volatile SPINTXD_REG* const spintxd_table[CH_MAX_COUNT] = {(SPINTXD_REG *)&SPI0TXD};
volatile SPINRXD_REG* const spinrxd_table[CH_MAX_COUNT] = {(SPINRXD_REG *)&SPI0RXD};
volatile SPININTF_REG* const spinintf_table[CH_MAX_COUNT] = {(SPININTF_REG *)&SPI0INTF};
volatile SPININTE_REG* const spininte_table[CH_MAX_COUNT] = {(SPININTE_REG *)&SPI0INTE};
#endif

static void setSpiaPortCh0(unsigned short spiMode);

#if defined(C17_MCUSEL_M01) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25) || \
	defined(C17_MCUSEL_589) || \
	defined(C17_MCUSEL_M40)
static void setSpiaPortCh1(unsigned short spiMode);
#endif

#if defined(C17_MCUSEL_W16)
static void setSpiaPortCh2(unsigned short spiMode);
#endif

/************************************************************************//**
 * @brief   initialize SPIA.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17initSpia(void)
{
	/// SPI CH 0 Setting
	if((SPICH0_INIT & 0x0200) == 0x0200)
	{
		c17setSpia(0, (SPICH0_INIT & 0x0100)>>8, (SPICH0_INIT & 0x0080)>>7, (SPICH0_INIT & 0x0060)>>5, (SPICH0_INIT & 0x0010)>>4, (SPICH0_INIT & 0x000f));

		setSpiaPortCh0((SPICH0_INIT & 0x0100)>>8);

		/// Set interrupt level.
		ILVSPIA_0 = INTERRUPT_LEVEL_SPIA0;
	}

#if defined(C17_MCUSEL_M01) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25) || \
	defined(C17_MCUSEL_589) || \
	defined(C17_MCUSEL_M40)
	/// SPI CH 1 Setting
	if((SPICH1_INIT & 0x0200) == 0x0200)
	{
		c17setSpia(1, (SPICH1_INIT & 0x0100)>>8, (SPICH1_INIT & 0x0080)>>7, (SPICH1_INIT & 0x0060)>>5, (SPICH1_INIT & 0x0010)>>4, (SPICH1_INIT & 0x000f));

		setSpiaPortCh1((SPICH1_INIT & 0x0100)>>8);

		/// Set interrupt level.
		ILVSPIA_1 = INTERRUPT_LEVEL_SPIA1;
	}
#endif

#if defined(C17_MCUSEL_W16)
	/// SPI CH 1 Setting
	if((SPICH2_INIT & 0x0200) == 0x0200)
	{
		c17setSpia(2, (SPICH2_INIT & 0x0100)>>8, (SPICH2_INIT & 0x0080)>>7, (SPICH2_INIT & 0x0060)>>5, (SPICH2_INIT & 0x0010)>>4, (SPICH2_INIT & 0x000f));

		setSpiaPortCh2((SPICH2_INIT & 0x0100)>>8);

		/// Set interrupt level.
		ILVSPIA_2 = INTERRUPT_LEVEL_SPIA2;
	}
#endif
}


/************************************************************************//**
 * @brief   initialize SPIA.
 *
 * @param   spiChannel    SPI CH
 * @param   spiMode
 *  @li @c      SPIA_MODE_MASTER    Master mode.
 *  @li @c      SPIA_MODE_SLAVE     Slave mode.
 * @param   spiPullupdown
 *  @li @c      C17_SPIA_PULL_DISABLE    pull-up or pull-down function disable.
 *  @li @c      C17_SPIA_PULL_ENABLE     pull-up or pull-down function enable.
 * @param   spiProtocol
 *  @li @c      C17_SPIA_MODE0    CPOL=0, CPHA=0.
 *  @li @c      C17_SPIA_MODE1    CPOL=0, CPHA=1.
 *  @li @c      C17_SPIA_MODE2    CPOL=1, CPHA=0.
 *  @li @c      C17_SPIA_MODE3    CPOL=1, CPHA=1.
 * @param   spiFormat
 *  @li @c      C17_SPIA_MSB    MSB first
 *  @li @c      C17_SPIA_LSB    LSB first
 * @param   spiBit
 *  @li @c      C17_SPIA_DATA_BIT2 to C17_SPIA_DATA_BIT16    Length of transfer data.
 *
 * @retval  none
 ***************************************************************************/
void c17setSpia(unsigned short spiChannel, unsigned short spiMode, unsigned short spiPullupdown, unsigned short spiProtocol, unsigned short spiFormat, unsigned short spiBit)
{
	SPINMOD_REG		spinmod_set;

	if(spiChannel >= CH_MAX_COUNT) {
		return;
	}

	/// Stop SPIA.
	c17stopSpia(spiChannel);

	/// Setting SPIA ch.N.
	/*  SPINMOD:SPIA Ch.N Mode Register */
	{
		spinmod_set.reg = 0;
		spinmod_set.bit.CHLN = spiBit;			/// - length
		spinmod_set.bit.PUEN = spiPullupdown;	/// - pull-up
		/**
		 * @todo Change NOCLKDIV.
		 */
		spinmod_set.bit.NOCLKDIV = 1;			/// - SPICLK0   : 16-bit timer operating clock frequency.
		spinmod_set.bit.LSBFST = spiFormat;		/// - Format

		switch(spiProtocol) {
		case C17_SPIA_MODE0:
			spinmod_set.bit.CPHA = 0;			/// - Phase     : 0.
			spinmod_set.bit.CPOL = 0;			/// - Polarity  : Active Low.
			break;
		case C17_SPIA_MODE1:
			spinmod_set.bit.CPHA = 1;			/// - Phase     : 1.
			spinmod_set.bit.CPOL = 0;			/// - Polarity  : Active High.
			break;
		case C17_SPIA_MODE2:
			spinmod_set.bit.CPHA = 0;			/// - Phase     : 0.
			spinmod_set.bit.CPOL = 1;			/// - Polarity  : Active High.
			break;
		case C17_SPIA_MODE3:
			spinmod_set.bit.CPHA = 1;			/// - Phase     : 1.
			spinmod_set.bit.CPOL = 1;			/// - Polarity  : Active Low.
			break;
		}
		spinmod_set.bit.MST = spiMode;			/// - Mode      : Master(1),Slave(0)
		spinmod_table[spiChannel]->reg = spinmod_set.reg;
	}
}


/************************************************************************//**
 * @brief   Setting Port(GPIO) for SPI CH.0.
 *
 * @param   spiMode
 *  @li @c      SPIA_MODE_MASTER    Master mode.
 *  @li @c      SPIA_MODE_SLAVE     Slave mode.
 *
 * @retval  none
 ***************************************************************************/
static void setSpiaPortCh0(unsigned short spiMode)
{
#if defined(C17_MCUSEL_M01) || \
	defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25) || \
	defined(C17_MCUSEL_589) || \
	defined(C17_MCUSEL_M10) || \
	defined(C17_MCUSEL_M40) || \
	defined(C17_MCUSEL_F63)
	P0MODSEL_REG	p0modsel_set;
	P0FNCSEL_REG	p0fncsel_set;

	P0DAT_REG		p0dat_set;
	P0IOEN_REG		p0ioen_set;
#endif

#if defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_589) || \
	defined(C17_MCUSEL_M10) || \
	defined(C17_MCUSEL_M40) || \
	defined(C17_MCUSEL_F63)
	P0UPMUX2_REG	p0upmux2_set;
	P0UPMUX3_REG	p0upmux3_set;
#endif

#if defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25)
	P0UPMUX0_REG	p0upmux0_set;
	P0UPMUX1_REG	p0upmux1_set;
#endif

#if defined(C17_MCUSEL_W15)
	P1MODSEL_REG	p1modsel_set;
	P1FNCSEL_REG	p1fncsel_set;
	P1UPMUX2_REG	p1upmux2_set;
	P1UPMUX3_REG	p1upmux3_set;
	P1DAT_REG		p1dat_set;
	P1IOEN_REG		p1ioen_set;
#endif


#if defined(C17_MCUSEL_M01)
    /**
     * @todo change PORT
     *
     * P04=PORT(Master)/SPISS(Slave)
     * P05=SDI
     * P06=SDO
     * P07=SPICLK
     */
	/* P0MODSEL: P0 Port Mode Select Register */
	{
		/// P0 Port Mode Select Register
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P07SEL = 1;			/// - P07 : Enable peripheral I/O function.
		p0modsel_set.bit.P06SEL = 1;			/// - P06 : Enable peripheral I/O function.
		p0modsel_set.bit.P05SEL = 1;			/// - P05 : Enable peripheral I/O function.
		/// Select Mode.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p0modsel_set.bit.P04SEL = 0;		/// - P04 : Use GPIO function
		}
		else{
			p0modsel_set.bit.P04SEL = 1;		/// - P04 : Enable peripheral I/O function.
		}
		P0MODSEL = p0modsel_set.reg;
	}
	/* P0FNCSEL: P0 Port Function Select Register */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P07MUX		= 1;		/// - P07 : UPMUX.
		p0fncsel_set.bit.P06MUX		= 1;		/// - P06 : UPMUX.
		p0fncsel_set.bit.P05MUX		= 1;		/// - P05 : UPMUX.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p0fncsel_set.bit.P04MUX		= 0;	/// - P04 : GPIO.
		}
		else{
			p0fncsel_set.bit.P04MUX		= 1;	/// - P04 : UPMUX.
		}
		P0FNCSEL = p0fncsel_set.reg;
	}
#endif

#if defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_589) || \
	defined(C17_MCUSEL_M10) || \
	defined(C17_MCUSEL_M40) || \
	defined(C17_MCUSEL_F63)
    /**
     * @todo change PORT
     *
     * P04=PORT(Master)/SPISS(Slave)
     * P05=SDI
     * P06=SDO
     * P07=SPICLK
     */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P0MODSEL: P0 Port Mode Select Register */
	{
		/// P0 Port Mode Select Register
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P07SEL = 1;			/// - P07 : Enable peripheral I/O function.
		p0modsel_set.bit.P06SEL = 1;			/// - P06 : Enable peripheral I/O function.
		p0modsel_set.bit.P05SEL = 1;			/// - P05 : Enable peripheral I/O function.
		/// Select Mode.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p0modsel_set.bit.P04SEL = 0;		/// - P04 : Use GPIO function
		}
		else{
			p0modsel_set.bit.P04SEL = 1;		/// - P04 : Enable peripheral I/O function.
		}
		P0MODSEL = p0modsel_set.reg;
	}
	/* P0FNCSEL: P0 Port Function Select Register */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P07MUX		= 1;		/// - P07 : UPMUX.
		p0fncsel_set.bit.P06MUX		= 1;		/// - P06 : UPMUX.
		p0fncsel_set.bit.P05MUX		= 1;		/// - P05 : UPMUX.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p0fncsel_set.bit.P04MUX		= 0;	/// - P04 : GPIO.
		}
		else{
			p0fncsel_set.bit.P04MUX		= 1;	/// - P04 : UPMUX.
		}
		P0FNCSEL = p0fncsel_set.reg;
	}

	/// Assign the SPIA input/output function to the ports.
	/* P0UPMUX2:P04-05 Universal Port Multiplexer Setting Register */
	{
		p0upmux2_set.reg = P0UPMUX2;
		p0upmux2_set.bit.P05PPFNC = 1;			/// - P05 : SDI.
		p0upmux2_set.bit.P05PERICH = 0;			/// - P05 : Ch.0.
		p0upmux2_set.bit.P05PERISEL = 2;		/// - P05 : SPIA.
		p0upmux2_set.bit.P04PPFNC = 4;			/// - P04 : SPISS.
		p0upmux2_set.bit.P04PERICH = 0;			/// - P04 : Ch.0.
		p0upmux2_set.bit.P04PERISEL = 2;		/// - P04 : SPIA.
		P0UPMUX2 = p0upmux2_set.reg;
	}
	/* P0UPMUX3: P06-07 Universal Port Multiplexer Setting Register */
	{
		p0upmux3_set.reg = P0UPMUX3;
		p0upmux3_set.bit.P07PPFNC = 3;			/// - P07 : SPICLK.
		p0upmux3_set.bit.P07PERICH = 0;			/// - P07 : Ch.0.
		p0upmux3_set.bit.P07PERISEL = 2;		/// - P07 : SPIA.
		p0upmux3_set.bit.P06PPFNC= 2;			/// - P06 : SDO.
		p0upmux3_set.bit.P06PERICH = 0;			/// - P06 : Ch.0.
		p0upmux3_set.bit.P06PERISEL	 = 2;		/// - P06 : SPIA.
		P0UPMUX3 = p0upmux3_set.reg;
	}
#endif

#if defined(C17_MCUSEL_M01) || \
	defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_589) || \
	defined(C17_MCUSEL_M10) || \
	defined(C17_MCUSEL_M40) || \
	defined(C17_MCUSEL_F63)

	if (spiMode == C17_SPIA_MODE_MASTER) {
		/// - P04 : GPIO (SPISS0).
		/// - P04 : High.
		/* P0DAT:P0 Port Data Register */
		{
			p0dat_set.reg = P0DAT;
			p0dat_set.bit.P04OUT = 1;
			P0DAT = p0dat_set.reg;
		}
		/// - P04 : Enable(Out).
		/* P0IOEN:P0 Port Enable Register */
		{
			p0ioen_set.reg = P0IOEN;
			p0ioen_set.bit.P04OEN = 1;
			P0IOEN = p0ioen_set.reg;
		}
	}
#endif

#if defined(C17_MCUSEL_W15)
    /**
     * @todo change PORT
     *
     * P14=PORT(Master)/SPISS(Slave)
     * P15=SDI
     * P16=SDO
     * P17=SPICLK
     */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P1MODSEL: P1 Port Mode Select Register */
	{
		/// P1 Port Mode Select Register
		p1modsel_set.reg = P1MODSEL;
		p1modsel_set.bit.P17SEL = 1;			/// - P17 : Enable peripheral I/O function.
		p1modsel_set.bit.P16SEL = 1;			/// - P16 : Enable peripheral I/O function.
		p1modsel_set.bit.P15SEL = 1;			/// - P15 : Enable peripheral I/O function.
		/// Select Mode.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p1modsel_set.bit.P14SEL = 0;		/// - P14 : Use GPIO function
		}
		else{
			p1modsel_set.bit.P14SEL = 1;		/// - P14 : Enable peripheral I/O function.
		}
		P1MODSEL = p1modsel_set.reg;
	}
	/* P1FNCSEL: P1 Port Function Select Register */
	{
		p1fncsel_set.reg = P1FNCSEL;
		p1fncsel_set.bit.P17MUX		= 1;		/// - P17 : UPMUX.
		p1fncsel_set.bit.P16MUX		= 1;		/// - P16 : UPMUX.
		p1fncsel_set.bit.P15MUX		= 1;		/// - P15 : UPMUX.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p1fncsel_set.bit.P14MUX		= 0;	/// - P14 : GPIO.
		}
		else{
			p1fncsel_set.bit.P14MUX		= 1;	/// - P14 : UPMUX.
		}
		P1FNCSEL = p1fncsel_set.reg;
	}
	/// Assign the SPI input/output function to the ports.
	/* P1UPMUX2:P14-15 Universal Port Multiplexer Setting Register */
	{
		p1upmux2_set.reg = P1UPMUX2;
		p1upmux2_set.bit.P15PPFNC = 1;			/// - P15 : SDI.
		p1upmux2_set.bit.P15PERICH = 0;			/// - P15 : Ch.0.
		p1upmux2_set.bit.P15PERISEL = 2;		/// - P15 : SPIA.
		p1upmux2_set.bit.P14PPFNC = 4;			/// - P14 : SPISS.
		p1upmux2_set.bit.P14PERICH = 0;			/// - P14 : Ch.0.
		p1upmux2_set.bit.P14PERISEL = 2;		/// - P14 : SPIA.
		P1UPMUX2 = p1upmux2_set.reg;
	}
	/* P1UPMUX3: P16-17 Universal Port Multiplexer Setting Register */
	{
		p1upmux3_set.reg = P1UPMUX3;
		p1upmux3_set.bit.P17PPFNC = 3;			/// - P17 : SPICLK.
		p1upmux3_set.bit.P17PERICH = 0;			/// - P17 : Ch.0.
		p1upmux3_set.bit.P17PERISEL = 2;		/// - P17 : SPIA.
		p1upmux3_set.bit.P16PPFNC= 2;			/// - P16 : SDO.
		p1upmux3_set.bit.P16PERICH = 0;			/// - P16 : Ch.0.
		p1upmux3_set.bit.P16PERISEL	 = 2;		/// - P16 : SPIA.
		P1UPMUX3 = p1upmux3_set.reg;
	}
	if (spiMode == C17_SPIA_MODE_MASTER) {
		/// - P14 : GPIO (SPISS0).
		/// - P14 : High.
		/* P1DAT:P1 Port Data Register */
		{
			p1dat_set.reg = P1DAT;
			p1dat_set.bit.P14OUT = 1;
			P1DAT = p1dat_set.reg;
		}
		/// - P14 : Enable(Out).
		/* P1IOEN:P1 Port Enable Register */
		{
			p1ioen_set.reg = P1IOEN;
			p1ioen_set.bit.P14OEN = 1;
			P1IOEN = p1ioen_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25)
    /**
     * @todo change PORT
     *
     * P00=PORT(Master)/SPISS(Slave)
     * P01=SDI
     * P02=SDO
     * P03=SPICLK
     */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P0MODSEL: P0 Port Mode Select Register */
	{
		/// P0 Port Mode Select Register
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P03SEL = 1;			/// - P03 : Enable peripheral I/O function.
		p0modsel_set.bit.P02SEL = 1;			/// - P02 : Enable peripheral I/O function.
		p0modsel_set.bit.P01SEL = 1;			/// - P01 : Enable peripheral I/O function.
		/// Select Mode.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p0modsel_set.bit.P00SEL = 0;		/// - P00 : Use GPIO function
		}
		else{
			p0modsel_set.bit.P00SEL = 1;		/// - P00: Enable peripheral I/O function.
		}
		P0MODSEL = p0modsel_set.reg;
	}
	/* P0FNCSEL: P0 Port Function Select Register */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P03MUX		= 1;		/// - P03 : UPMUX.
		p0fncsel_set.bit.P02MUX		= 1;		/// - P02 : UPMUX.
		p0fncsel_set.bit.P01MUX		= 1;		/// - P01 : UPMUX.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p0fncsel_set.bit.P00MUX		= 0;	/// - P00 : GPIO.
		}
		else{
			p0fncsel_set.bit.P00MUX		= 1;	/// - P00 : UPMUX.
		}
		P0FNCSEL = p0fncsel_set.reg;
	}

	/// Assign the SPIA input/output function to the ports.
	/* P0UPMUX0:P00-01 Universal Port Multiplexer Setting Register */
	{
		p0upmux0_set.reg = P0UPMUX0;
		p0upmux0_set.bit.P01PPFNC = 1;			/// - P01 : SDI.
		p0upmux0_set.bit.P01PERICH = 0;			/// - P01 : Ch.0.
		p0upmux0_set.bit.P01PERISEL = 2;		/// - P01 : SPIA.
		p0upmux0_set.bit.P00PPFNC = 4;			/// - P00 : SPISS.
		p0upmux0_set.bit.P00PERICH = 0;			/// - P00 : Ch.0.
		p0upmux0_set.bit.P00PERISEL = 2;		/// - P00 : SPIA.
		P0UPMUX0 = p0upmux0_set.reg;
	}
	/* P0UPMUX1: P02-03 Universal Port Multiplexer Setting Register */
	{
		p0upmux1_set.reg = P0UPMUX1;
		p0upmux1_set.bit.P03PPFNC = 3;			/// - P03 : SPICLK.
		p0upmux1_set.bit.P03PERICH = 0;			/// - P03 : Ch.0.
		p0upmux1_set.bit.P03PERISEL = 2;		/// - P03 : SPIA.
		p0upmux1_set.bit.P02PPFNC= 2;			/// - P02 : SDO.
		p0upmux1_set.bit.P02PERICH = 0;			/// - P02 : Ch.0.
		p0upmux1_set.bit.P02PERISEL	 = 2;		/// - P02 : SPIA.
		P0UPMUX1 = p0upmux1_set.reg;
	}

	if (spiMode == C17_SPIA_MODE_MASTER) {
		/// - P00 : GPIO (SPISS0).
		/// - P00 : High.
		/* P0DAT:P0 Port Data Register */
		{
			p0dat_set.reg = P0DAT;
			p0dat_set.bit.P00OUT = 1;
			P0DAT = p0dat_set.reg;
		}
		/// - P00 : Enable(Out).
		/* P0IOEN:P0 Port Enable Register */
		{
			p0ioen_set.reg = P0IOEN;
			p0ioen_set.bit.P00OEN = 1;
			P0IOEN = p0ioen_set.reg;
		}
	}
#endif
}


#if defined(C17_MCUSEL_M01) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || \
	defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25) || \
	defined(C17_MCUSEL_589) || \
	defined(C17_MCUSEL_M40)
/************************************************************************//**
 * @brief   Setting Port(GPIO) for SPI CH.1.
 *
 * @param   spiMode
 *  @li @c      SPIA_MODE_MASTER    Master mode.
 *  @li @c      SPIA_MODE_SLAVE     Slave mode.
 *
 * @retval  none
 ***************************************************************************/
static void setSpiaPortCh1(unsigned short spiMode)
{
#if defined(C17_MCUSEL_M01) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_589)
	P3MODSEL_REG	p3modsel_set;
	P3FNCSEL_REG	p3fncsel_set;

#if defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_589)
	P3UPMUX0_REG	p3upmux0_set;
#endif
#if defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_589)
	P3UPMUX1_REG	p3upmux1_set;
	P3DAT_REG		p3dat_set;
	P3IOEN_REG		p3ioen_set;
#endif
#if defined(C17_MCUSEL_W23)
	P3UPMUX2_REG	p3upmux2_set;
	P3UPMUX3_REG	p3upmux3_set;
#endif
#if defined(C17_MCUSEL_M01)
	P0MODSEL_REG	p0modsel_set;
	P0FNCSEL_REG	p0fncsel_set;
	P0DAT_REG		p0dat_set;
	P0IOEN_REG		p0ioen_set;
#endif
#endif

#if defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_M40)
	P1MODSEL_REG	p1modsel_set;
	P1FNCSEL_REG	p1fncsel_set;
	P1UPMUX2_REG	p1upmux2_set;
	P1UPMUX3_REG	p1upmux3_set;

	P1DAT_REG		p1dat_set;
	P1IOEN_REG		p1ioen_set;
#endif

#if defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25)
	P2MODSEL_REG	p2modsel_set;
	P2FNCSEL_REG	p2fncsel_set;
	P2UPMUX2_REG	p2upmux2_set;
	P2UPMUX3_REG	p2upmux3_set;

	P2DAT_REG		p2dat_set;
	P2IOEN_REG		p2ioen_set;
#endif

#if defined(C17_MCUSEL_M01)
    /**
     * P33=SDI
     * P34=SDO
     * P35=SPICLK
     *
     * @todo change PORT
     *
     * P36=SPISS(Slave)
     * P00=PORT(Master)
     */
	/* P0MODSEL: P0 Port Mode Select Register */
	/* P3MODSEL: P3 Port Mode Select Register */
	{
		/// P3 Port Mode Select Register
		p0modsel_set.reg = P0MODSEL;
		p3modsel_set.reg = P3MODSEL;
		p3modsel_set.bit.P35SEL = 1;			/// - P35 : Enable peripheral I/O function.
		p3modsel_set.bit.P34SEL = 1;			/// - P34 : Enable peripheral I/O function.
		p3modsel_set.bit.P33SEL = 1;			/// - P33 : Enable peripheral I/O function.
		/// Select Mode.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p0modsel_set.bit.P00SEL = 0;		/// - P00 : Use GPIO function
		}
		else{
			p3modsel_set.bit.P36SEL = 1;		/// - P36 : Enable peripheral I/O function.
		}
		P0MODSEL = p0modsel_set.reg;
		P3MODSEL = p3modsel_set.reg;
	}
	/* P0FNCSEL: P0 Port Function Select Register */
	/* P3FNCSEL: P3 Port Function Select Register */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p3fncsel_set.reg = P3FNCSEL;
		p3fncsel_set.bit.P35MUX		= 1;		/// - P35 : SPICLK.
		p3fncsel_set.bit.P34MUX		= 1;		/// - P34 : SDO.
		p3fncsel_set.bit.P33MUX		= 1;		/// - P33 : SDI.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p0fncsel_set.bit.P00MUX		= 0;	/// - P00 : GPIO.
		}
		else{
			p3fncsel_set.bit.P36MUX		= 1;	/// - P36 : SPISS.
		}
		P0FNCSEL = p0fncsel_set.reg;
		P3FNCSEL = p3fncsel_set.reg;
	}
	if (spiMode == C17_SPIA_MODE_MASTER) {
		/// - P00 : GPIO.
		/// - P00 : High.
		/* P0DAT:P0 Port Data Register */
		{
			p0dat_set.reg = P0DAT;
			p0dat_set.bit.P00OUT = 1;
			P0DAT = p0dat_set.reg;
		}
		/// - P00 : Enable(Out).
		/* P0IOEN:P0 Port Enable Register */
		{
			p0ioen_set.reg = P0IOEN;
			p0ioen_set.bit.P00OEN = 1;
			P0IOEN = p0ioen_set.reg;
		}
	}
#endif

#if defined(C17_MCUSEL_W23)
    /**
     * @todo change PORT
     *
     * P33=SDI
     * P34=SDO
     * P35=SPICLK
     * P36=PORT(Master)/SPISS(Slave)
     */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P3MODSEL: P3 Port Mode Select Register */
	{
		/// P3 Port Mode Select Register
		p3modsel_set.reg = P3MODSEL;
		p3modsel_set.bit.P35SEL = 1;			/// - P35 : Enable peripheral I/O function.
		p3modsel_set.bit.P34SEL = 1;			/// - P34 : Enable peripheral I/O function.
		p3modsel_set.bit.P33SEL = 1;			/// - P33 : Enable peripheral I/O function.
		/// Select Mode.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p3modsel_set.bit.P36SEL = 0;		/// - P36 : Use GPIO function
		}
		else{
			p3modsel_set.bit.P36SEL = 1;		/// - P36 : Enable peripheral I/O function.
		}
		P3MODSEL = p3modsel_set.reg;
	}
	/* P3FNCSEL: P3 Port Function Select Register */
	{
		p3fncsel_set.reg = P3FNCSEL;
		p3fncsel_set.bit.P35MUX		= 1;		/// - P35 : UPMUX.
		p3fncsel_set.bit.P34MUX		= 1;		/// - P34 : UPMUX.
		p3fncsel_set.bit.P33MUX		= 1;		/// - P33 : UPMUX.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p3fncsel_set.bit.P36MUX		= 0;	/// - P36 : GPIO.
		}
		else{
			p3fncsel_set.bit.P36MUX		= 1;	/// - P36 : UPMUX.
		}
		P3FNCSEL = p3fncsel_set.reg;
	}
	/// Assign the SPI input/output function to the ports.
	/* P3UPMUX1:P32-33 Universal Port Multiplexer Setting Register */
	{
		p3upmux1_set.reg = P3UPMUX1;
		p3upmux1_set.bit.P33PPFNC = 1;			/// - P33 : SDI.
		p3upmux1_set.bit.P33PERICH = 1;			/// - P33 : Ch.1.
		p3upmux1_set.bit.P33PERISEL = 2;		/// - P33 : SPIA.
		P3UPMUX1 = p3upmux1_set.reg;
	}
	/// Assign the SPI input/output function to the ports.
	/* P3UPMUX2:P34-35 Universal Port Multiplexer Setting Register */
	{
		p3upmux2_set.reg = P3UPMUX2;
		p3upmux2_set.bit.P35PPFNC = 3;			/// - P35 : SPICLK.
		p3upmux2_set.bit.P35PERICH = 1;			/// - P35 : Ch.1.
		p3upmux2_set.bit.P35PERISEL = 2;		/// - P35 : SPIA.
		p3upmux2_set.bit.P34PPFNC = 2;			/// - P34 : SDO.
		p3upmux2_set.bit.P34PERICH = 1;			/// - P34 : Ch.1.
		p3upmux2_set.bit.P34PERISEL = 2;		/// - P34 : SPIA.
		P3UPMUX2 = p3upmux2_set.reg;
	}
	/* P3UPMUX3: P36-37 Universal Port Multiplexer Setting Register */
	{
		p3upmux3_set.reg = P3UPMUX3;
		p3upmux3_set.bit.P36PPFNC = 4;			/// - P36 : SPISS.
		p3upmux3_set.bit.P36PERICH = 1;			/// - P36 : Ch.1.
		p3upmux3_set.bit.P36PERISEL = 2;		/// - P36 : SPIA.
		P3UPMUX3 = p3upmux3_set.reg;
	}
	if (spiMode == C17_SPIA_MODE_MASTER) {
		/// - P36 : GPIO (SPISS1).
		/// - P36 : High.
		/* P3DAT:P3 Port Data Register */
		{
			p3dat_set.reg = P3DAT;
			p3dat_set.bit.P36OUT = 1;
			P3DAT = p3dat_set.reg;
		}
		/// - P36 : Enable(Out).
		/* P3IOEN:P3 Port Enable Register */
		{
			p3ioen_set.reg = P3IOEN;
			p3ioen_set.bit.P36OEN = 1;
			P3IOEN = p3ioen_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_M40)
    /**
     * @todo change PORT
     *
     * P14=PORT(Master)/SPISS(Slave)
     * P15=SDI
     * P16=SDO
     * P17=SPICLK
     */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P1MODSEL: P1 Port Mode Select Register */
	{
		/// P1 Port Mode Select Register
		p1modsel_set.reg = P1MODSEL;
		p1modsel_set.bit.P17SEL = 1;			/// - P17 : Enable peripheral I/O function.
		p1modsel_set.bit.P16SEL = 1;			/// - P16 : Enable peripheral I/O function.
		p1modsel_set.bit.P15SEL = 1;			/// - P15 : Enable peripheral I/O function.
		/// Select Mode.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p1modsel_set.bit.P14SEL = 0;		/// - P14 : Use GPIO function
		}
		else{
			p1modsel_set.bit.P14SEL = 1;		/// - P14 : Enable peripheral I/O function.
		}
		P1MODSEL = p1modsel_set.reg;
	}
	/* P1FNCSEL: P1 Port Function Select Register */
	{
		p1fncsel_set.reg = P1FNCSEL;
		p1fncsel_set.bit.P17MUX		= 1;		/// - P17 : UPMUX.
		p1fncsel_set.bit.P16MUX		= 1;		/// - P16 : UPMUX.
		p1fncsel_set.bit.P15MUX		= 1;		/// - P15 : UPMUX.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p1fncsel_set.bit.P14MUX		= 0;	/// - P14 : GPIO.
		}
		else{
			p1fncsel_set.bit.P14MUX		= 1;	/// - P14 : UPMUX.
		}
		P1FNCSEL = p1fncsel_set.reg;
	}
	/// Assign the SPI input/output function to the ports.
	/* P1UPMUX2:P14-15 Universal Port Multiplexer Setting Register */
	{
		p1upmux2_set.reg = P1UPMUX2;
		p1upmux2_set.bit.P15PPFNC = 1;			/// - P15 : SDI.
		p1upmux2_set.bit.P15PERICH = 1;			/// - P15 : Ch.1.
		p1upmux2_set.bit.P15PERISEL = 2;		/// - P15 : SPIA.
		p1upmux2_set.bit.P14PPFNC = 4;			/// - P14 : SPISS.
		p1upmux2_set.bit.P14PERICH = 1;			/// - P14 : Ch.1.
		p1upmux2_set.bit.P14PERISEL = 2;		/// - P14 : SPIA.
		P1UPMUX2 = p1upmux2_set.reg;
	}
	/* P1UPMUX3: P16-17 Universal Port Multiplexer Setting Register */
	{
		p1upmux3_set.reg = P1UPMUX3;
		p1upmux3_set.bit.P17PPFNC = 3;			/// - P17 : SPICLK.
		p1upmux3_set.bit.P17PERICH = 1;			/// - P17 : Ch.1.
		p1upmux3_set.bit.P17PERISEL = 2;		/// - P17 : SPIA.
		p1upmux3_set.bit.P16PPFNC= 2;			/// - P16 : SDO.
		p1upmux3_set.bit.P16PERICH = 1;			/// - P16 : Ch.1.
		p1upmux3_set.bit.P16PERISEL	 = 2;		/// - P16 : SPIA.
		P1UPMUX3 = p1upmux3_set.reg;
	}
	if (spiMode == C17_SPIA_MODE_MASTER) {
		/// - P14 : GPIO (SPISS1).
		/// - P14 : High.
		/* P1DAT:P1 Port Data Register */
		{
			p1dat_set.reg = P1DAT;
			p1dat_set.bit.P14OUT = 1;
			P1DAT = p1dat_set.reg;
		}
		/// - P14 : Enable(Out).
		/* P1IOEN:P1 Port Enable Register */
		{
			p1ioen_set.reg = P1IOEN;
			p1ioen_set.bit.P14OEN = 1;
			P1IOEN = p1ioen_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_589)
    /**
     * @todo change PORT
     *
     * P30=PORT(Master)/SPISS(Slave)
     * P31=SDI
     * P32=SDO
     * P33=SPICLK
     */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P3MODSEL: P3 Port Mode Select Register */
	{
		/// P3 Port Mode Select Register
		p3modsel_set.reg = P3MODSEL;
		p3modsel_set.bit.P31SEL = 1;			/// - P31 : Enable peripheral I/O function.
		p3modsel_set.bit.P32SEL = 1;			/// - P32 : Enable peripheral I/O function.
		p3modsel_set.bit.P33SEL = 1;			/// - P33 : Enable peripheral I/O function.
		/// Select Mode.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p3modsel_set.bit.P30SEL = 0;		/// - P30 : Use GPIO function
		}
		else{
			p3modsel_set.bit.P30SEL = 1;		/// - P30 : Enable peripheral I/O function.
		}
		P3MODSEL = p3modsel_set.reg;
	}
	/* P0FNCSEL: P3 Port Function Select Register */
	{
		p3fncsel_set.reg = P3FNCSEL;
		p3fncsel_set.bit.P31MUX		= 1;		/// - P31 : UPMUX.
		p3fncsel_set.bit.P32MUX		= 1;		/// - P32 : UPMUX.
		p3fncsel_set.bit.P33MUX		= 1;		/// - P33 : UPMUX.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p3fncsel_set.bit.P30MUX		= 0;	/// - P30 : GPIO.
		}
		else{
			p3fncsel_set.bit.P30MUX		= 1;	/// - P30 : UPMUX.
		}
		P3FNCSEL = p3fncsel_set.reg;
	}

	/// Assign the SPIA input/output function to the ports.
	/* P3UPMUX0:P30-31 Universal Port Multiplexer Setting Register */
	{
		p3upmux0_set.reg = P3UPMUX0;
		p3upmux0_set.bit.P31PPFNC = 1;			/// - P31 : SDI.
		p3upmux0_set.bit.P31PERICH = 1;			/// - P31 : Ch.1.
		p3upmux0_set.bit.P31PERISEL = 2;		/// - P31 : SPIA.
		p3upmux0_set.bit.P30PPFNC = 4;			/// - P30 : SPISS.
		p3upmux0_set.bit.P30PERICH = 1;			/// - P30 : Ch.1.
		p3upmux0_set.bit.P30PERISEL = 2;		/// - P30 : SPIA.
		P3UPMUX0 = p3upmux0_set.reg;
	}
	/* P3UPMUX1: P32-33 Universal Port Multiplexer Setting Register */
	{
		p3upmux1_set.reg = P3UPMUX1;
		p3upmux1_set.bit.P33PPFNC = 3;			/// - P33 : SPICLK.
		p3upmux1_set.bit.P33PERICH = 1;			/// - P33 : Ch.1.
		p3upmux1_set.bit.P33PERISEL = 2;		/// - P33 : SPIA.
		p3upmux1_set.bit.P32PPFNC= 2;			/// - P32 : SDO.
		p3upmux1_set.bit.P32PERICH = 1;			/// - P32 : Ch.1.
		p3upmux1_set.bit.P32PERISEL	 = 2;		/// - P32 : SPIA.
		P3UPMUX1 = p3upmux1_set.reg;
	}

	if (spiMode == C17_SPIA_MODE_MASTER) {
		/// - P30 : GPIO (SPISS0).
		/// - P30 : High.
		/* P3DAT:P3 Port Data Register */
		{
			p3dat_set.reg = P3DAT;
			p3dat_set.bit.P30OUT = 1;
			P3DAT = p3dat_set.reg;
		}
		/// - P30 : Enable(Out).
		/* P3IOEN:P3 Port Enable Register */
		{
			p3ioen_set.reg = P3IOEN;
			p3ioen_set.bit.P30OEN = 1;
			P3IOEN = p3ioen_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25)
    /**
     * @todo change PORT
     *
     * P24=PORT(Master)/SPISS(Slave)
     * P25=SDI
     * P26=SDO
     * P27=SPICLK
     */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P2MODSEL: P2 Port Mode Select Register */
	{
		/// P2 Port Mode Select Register
		p2modsel_set.reg = P2MODSEL;
		p2modsel_set.bit.P27SEL = 1;			/// - P27 : Enable peripheral I/O function.
		p2modsel_set.bit.P26SEL = 1;			/// - P26 : Enable peripheral I/O function.
		p2modsel_set.bit.P25SEL = 1;			/// - P25 : Enable peripheral I/O function.
		/// Select Mode.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p2modsel_set.bit.P24SEL = 0;		/// - P24 : Use GPIO function
		}
		else{
			p2modsel_set.bit.P24SEL = 1;		/// - P24 : Enable peripheral I/O function.
		}
		P2MODSEL = p2modsel_set.reg;
	}
	/* P2FNCSEL: P2 Port Function Select Register */
	{
		p2fncsel_set.reg = P2FNCSEL;
		p2fncsel_set.bit.P27MUX		= 1;		/// - P27 : UPMUX.
		p2fncsel_set.bit.P26MUX		= 1;		/// - P26 : UPMUX.
		p2fncsel_set.bit.P25MUX		= 1;		/// - P25 : UPMUX.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p2fncsel_set.bit.P24MUX		= 0;	/// - P24 : GPIO.
		}
		else{
			p2fncsel_set.bit.P24MUX		= 1;	/// - P24 : UPMUX.
		}
		P2FNCSEL = p2fncsel_set.reg;
	}
	/// Assign the SPI input/output function to the ports.
	/* P2UPMUX2:P24-25 Universal Port Multiplexer Setting Register */
	{
		p2upmux2_set.reg = P2UPMUX2;
		p2upmux2_set.bit.P25PPFNC = 1;			/// - P25 : SDI.
		p2upmux2_set.bit.P25PERICH = 1;			/// - P25 : Ch.1.
		p2upmux2_set.bit.P25PERISEL = 2;		/// - P25 : SPIA.
		p2upmux2_set.bit.P24PPFNC = 4;			/// - P24 : SPISS.
		p2upmux2_set.bit.P24PERICH = 1;			/// - P24 : Ch.1.
		p2upmux2_set.bit.P24PERISEL = 2;		/// - P24 : SPIA.
		P2UPMUX2 = p2upmux2_set.reg;
	}
	/* P2UPMUX3: P26-27 Universal Port Multiplexer Setting Register */
	{
		p2upmux3_set.reg = P2UPMUX3;
		p2upmux3_set.bit.P27PPFNC = 3;			/// - P27 : SPICLK.
		p2upmux3_set.bit.P27PERICH = 1;			/// - P27 : Ch.1.
		p2upmux3_set.bit.P27PERISEL = 2;		/// - P27 : SPIA.
		p2upmux3_set.bit.P26PPFNC= 2;			/// - P26 : SDO.
		p2upmux3_set.bit.P26PERICH = 1;			/// - P26 : Ch.1.
		p2upmux3_set.bit.P26PERISEL	 = 2;		/// - P26 : SPIA.
		P2UPMUX3 = p2upmux3_set.reg;
	}
	if (spiMode == C17_SPIA_MODE_MASTER) {
		/// - P24 : GPIO (SPISS1).
		/// - P24 : High.
		/* P2DAT:P2 Port Data Register */
		{
			p2dat_set.reg = P2DAT;
			p2dat_set.bit.P24OUT = 1;
			P2DAT = p2dat_set.reg;
		}
		/// - P24 : Enable(Out).
		/* P2IOEN:P2 Port Enable Register */
		{
			p2ioen_set.reg = P2IOEN;
			p2ioen_set.bit.P24OEN = 1;
			P2IOEN = p2ioen_set.reg;
		}
	}
#endif
}
#endif


#if defined(C17_MCUSEL_W16)
/************************************************************************//**
 * @brief   Setting Port(GPIO) for SPI CH.2.
 *
 * @param   spiMode
 *  @li @c      SPIA_MODE_MASTER    Master mode.
 *  @li @c      SPIA_MODE_SLAVE     Slave mode.
 *
 * @retval  none
 ***************************************************************************/
static void setSpiaPortCh2(unsigned short spiMode)
{

#if defined(C17_MCUSEL_W16)
	P2MODSEL_REG	p2modsel_set;
	P2FNCSEL_REG	p2fncsel_set;
	P2UPMUX0_REG	p2upmux0_set;
	P2UPMUX1_REG	p2upmux1_set;

	P2DAT_REG		p2dat_set;
	P2IOEN_REG		p2ioen_set;

#endif


#if defined(C17_MCUSEL_W16)
    /**
     * @todo change PORT
     *
     * P20=PORT(Master)/SPISS(Slave)
     * P21=SDI
     * P22=SDO
     * P23=SPICLK
     */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P2MODSEL: P2 Port Mode Select Register */
	{
		/// P2 Port Mode Select Register
		p2modsel_set.reg = P2MODSEL;
		p2modsel_set.bit.P23SEL = 1;			/// - P23 : Enable peripheral I/O function.
		p2modsel_set.bit.P22SEL = 1;			/// - P22 : Enable peripheral I/O function.
		p2modsel_set.bit.P21SEL = 1;			/// - P21 : Enable peripheral I/O function.
		/// Select Mode.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p2modsel_set.bit.P20SEL = 0;		/// - P20 : Use GPIO function
		}
		else{
			p2modsel_set.bit.P20SEL = 1;		/// - P20 : Enable peripheral I/O function.
		}
		P2MODSEL = p2modsel_set.reg;
	}
	/* P2FNCSEL: P2 Port Function Select Register */
	{
		p2fncsel_set.reg = P2FNCSEL;
		p2fncsel_set.bit.P23MUX		= 1;		/// - P23 : UPMUX.
		p2fncsel_set.bit.P22MUX		= 1;		/// - P22 : UPMUX.
		p2fncsel_set.bit.P21MUX		= 1;		/// - P21 : UPMUX.
		if (spiMode == C17_SPIA_MODE_MASTER) {
			p2fncsel_set.bit.P20MUX		= 0;	/// - P20 : GPIO.
		}
		else{
			p2fncsel_set.bit.P20MUX		= 1;	/// - P20 : UPMUX.
		}
		P2FNCSEL = p2fncsel_set.reg;
	}

	/// Assign the SPIA input/output function to the ports.
	/* P2UPMUX0:P20-21 Universal Port Multiplexer Setting Register */
	{
		p2upmux0_set.reg = P2UPMUX0;
		p2upmux0_set.bit.P21PPFNC = 1;			/// - P21 : SDI.
		p2upmux0_set.bit.P21PERICH = 2;			/// - P21 : Ch.0.
		p2upmux0_set.bit.P21PERISEL = 2;		/// - P21 : SPIA.
		p2upmux0_set.bit.P20PPFNC = 4;			/// - P20 : SPISS.
		p2upmux0_set.bit.P20PERICH = 2;			/// - P20 : Ch.0.
		p2upmux0_set.bit.P20PERISEL = 2;		/// - P20 : SPIA.
		P2UPMUX0 = p2upmux0_set.reg;
	}
	/* P2UPMUX1: P22-23 Universal Port Multiplexer Setting Register */
	{
		p2upmux1_set.reg = P2UPMUX1;
		p2upmux1_set.bit.P23PPFNC = 3;			/// - P23 : SPICLK.
		p2upmux1_set.bit.P23PERICH = 2;			/// - P23 : Ch.0.
		p2upmux1_set.bit.P23PERISEL = 2;		/// - P23 : SPIA.
		p2upmux1_set.bit.P22PPFNC= 2;			/// - P22 : SDO.
		p2upmux1_set.bit.P22PERICH = 2;			/// - P22 : Ch.0.
		p2upmux1_set.bit.P22PERISEL	 = 2;		/// - P22 : SPIA.
		P2UPMUX1 = p2upmux1_set.reg;
	}

	if (spiMode == C17_SPIA_MODE_MASTER) {
		/// - P20 : GPIO (SPISS2).
		/// - P20 : High.
		/* P2DAT:P2 Port Data Register */
		{
			p2dat_set.reg = P2DAT;
			p2dat_set.bit.P20OUT = 1;
			P2DAT = p2dat_set.reg;
		}
		/// - P20 : Enable(Out).
		/* P2IOEN:P2 Port Enable Register */
		{
			p2ioen_set.reg = P2IOEN;
			p2ioen_set.bit.P20OEN = 1;
			P2IOEN = p2ioen_set.reg;
		}
	}
#endif
}
#endif


/************************************************************************//**
 * @brief   Start SPIA.
 *
 * @param   spiChannel    SPI CH
 *
 * @retval  none
 ***************************************************************************/
void c17startSpia(unsigned short spiChannel)
{
	SPINCTL_REG		spinctl_set;
	SPININTF_REG	spinintf_set;

	/// This variable is for dummy reading.
	/// Warning comes out, but there is no problem.
	unsigned short dummy;

	if(spiChannel >= CH_MAX_COUNT) {
		return;
	}

	/// clear flag
	dummy = spinrxd_table[spiChannel]->reg ;		// RXD dummy read

	/// clear flag
	/* SPININTF_REG: SPIA Ch.N Interrupt Flag Register */
	{
		spinintf_set.reg = 0;
		spinintf_set.bit.OEIF = 1;
		spinintf_set.bit.TENDIF = 1;
		spinintf_table[spiChannel]->reg = spinintf_set.reg;
	}

	/// Execute software reset.
	/// Enable SPIA.
	/* SPI0CTL: SPIA Ch.0 Control Register */
	{
		spinctl_set.bit.SFTRST = 1;
		spinctl_set.bit.MODEN = 0;
		spinctl_table[spiChannel]->reg = spinctl_set.reg;

		spinctl_set.bit.SFTRST = 0;
		spinctl_set.bit.MODEN = 1;
		spinctl_table[spiChannel]->reg = spinctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   Stop SPIA.
 *
 * @param   spiChannel    SPI CH
 *
 * @retval  none
 ***************************************************************************/
void c17stopSpia(unsigned short spiChannel)
{
	SPINCTL_REG	spinctl_set;

	if(spiChannel >= CH_MAX_COUNT) {
		return;
	}

	/// Disable SPIA.
	/* SPINCTL: SPIA Ch.N Control Register */
	{
		spinctl_set.bit.SFTRST = 0;
		spinctl_set.bit.MODEN = 0;
		spinctl_table[spiChannel]->reg = spinctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   Assert SPIA(Master mode only).
 *
 * @param   spiChannel    SPI CH
 *
 * @retval  none
 ***************************************************************************/
void c17assertSpiaSs(unsigned short spiChannel)
{
#if defined(C17_MCUSEL_M01) || \
	defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25) || \
	defined(C17_MCUSEL_589) || \
	defined(C17_MCUSEL_M10) || \
	defined(C17_MCUSEL_M40) || \
	defined(C17_MCUSEL_F63)
	P0DAT_REG		p0dat_set;
#endif
#if defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W15) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_M40)
	P1DAT_REG		p1dat_set;
#endif
#if defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25)
	P2DAT_REG		p2dat_set;
#endif
#if defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_589)
	P3DAT_REG		p3dat_set;
#endif

#if defined(C17_MCUSEL_M01) || \
	defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_589) || \
	defined(C17_MCUSEL_M10) || \
	defined(C17_MCUSEL_M40) || \
	defined(C17_MCUSEL_F63)
	// Assert SPISS.
	if(spiChannel == 0)
	{
		/**
		 * @todo change PORT
		 *
		 * P04=PORT(Master)
		 */
		/// - P04 : LOW
		/* P0DAT:P0 Port Data Register */
		{
			p0dat_set.reg = P0DAT;
			p0dat_set.bit.P04OUT = 0;
			P0DAT = p0dat_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_W15)
	// Assert SPISS.
	if(spiChannel == 0)
	{
		/**
		 * @todo change PORT
		 *
		 * P14=PORT(Master)
		 */
		/// - P14 : LOW
		/* P1DAT:P1 Port Data Register */
		{
			p1dat_set.reg = P1DAT;
			p1dat_set.bit.P14OUT = 0;
			P1DAT = p1dat_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25)
	// Assert SPISS.
	if(spiChannel == 0)
	{
		/**
		 * @todo change PORT
		 *
		 * P00=PORT(Master)
		 */
		/// - P00 : LOW
		/* P0DAT:P0 Port Data Register */
		{
			p0dat_set.reg = P0DAT;
			p0dat_set.bit.P00OUT = 0;
			P0DAT = p0dat_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_M01)
	else if(spiChannel == 1)
	{
		/**
		 * @todo change PORT
		 *
		 * P00=PORT(Master)
		 */
		/// - P00 : LOW
		/* P0DAT:P0 Port Data Register */
		{
			p0dat_set.reg = P0DAT;
			p0dat_set.bit.P00OUT = 0;
			P0DAT = p0dat_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_W23)
	else if(spiChannel == 1)
	{
		/**
		 * @todo change PORT
		 *
		 * P36=PORT(Master)
		 */
		/// - P36 : LOW
		/* P3DAT:P3 Port Data Register */
		{
			p3dat_set.reg = P3DAT;
			p3dat_set.bit.P36OUT = 0;
			P3DAT = p3dat_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_M40)
	else if(spiChannel == 1)
	{
		/**
		 * @todo change PORT
		 *
		 * P14=PORT(Master)
		 */
		/// - P14 : LOW
		/* P1DAT:P1 Port Data Register */
		{
			p1dat_set.reg = P1DAT;
			p1dat_set.bit.P14OUT = 0;
			P1DAT = p1dat_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_589)
	else if(spiChannel == 1)
	{
		/**
		 * @todo change PORT
		 *
		 * P30=PORT(Master)
		 */
		/// - P30 : LOW
		/* P3DAT:P3 Port Data Register */
		{
			p3dat_set.reg = P3DAT;
			p3dat_set.bit.P30OUT = 0;
			P3DAT = p3dat_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25)
	else if(spiChannel == 1)
	{
		/**
		 * @todo change PORT
		 *
		 * P24=PORT(Master)
		 */
		/// - P24 : LOW
		/* P2DAT:P2 Port Data Register */
		{
			p2dat_set.reg = P2DAT;
			p2dat_set.bit.P24OUT = 0;
			P2DAT = p2dat_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_W16)
	else if(spiChannel == 2)
	{
		/**
		 * @todo change PORT
		 *
		 * P20=PORT(Master)
		 */
		/// - P20 : LOW
		/* P2DAT:P2 Port Data Register */
		{
			p2dat_set.reg = P2DAT;
			p2dat_set.bit.P20OUT = 0;
			P2DAT = p2dat_set.reg;
		}
	}
#endif
}


/************************************************************************//**
 * @brief   Negate SPIA(Master mode only).
 *
 * @param   spiChannel    SPI CH
 *
 * @retval  none
 ***************************************************************************/
void c17negateSpiaSs(unsigned short spiChannel)
{
#if defined(C17_MCUSEL_M01) || \
	defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) ||\
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25) || \
	defined(C17_MCUSEL_589) || \
	defined(C17_MCUSEL_M10) || \
	defined(C17_MCUSEL_M40) || \
	defined(C17_MCUSEL_F63)
	P0DAT_REG		p0dat_set;
#endif
#if defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W15) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_M40)
	P1DAT_REG		p1dat_set;
#endif
#if defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25)
	P2DAT_REG		p2dat_set;
#endif
#if defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_589)
	P3DAT_REG		p3dat_set;
#endif

#if defined(C17_MCUSEL_M01) || \
	defined(C17_MCUSEL_W22) || defined(C17_MCUSEL_W23) || \
	defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W12) || defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_589) || \
	defined(C17_MCUSEL_M10) || \
	defined(C17_MCUSEL_M40) || \
	defined(C17_MCUSEL_F63)
	/// Negate SPISS.
	if(spiChannel == 0)
	{
		/**
		 * @todo change PORT
		 *
		 * P04=PORT(Master)
		 */
		/// - P04 : High.
		/* P0DAT:P0 Port Data Register */
		{
			p0dat_set.reg = P0DAT;
			p0dat_set.bit.P04OUT = 1;
			P0DAT = p0dat_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_W15)
	if(spiChannel == 0)
	{
		/**
		 * @todo change PORT
		 *
		 * P14=PORT(Master)
		 */
		/// - P14 : High.
		/* P1DAT:P1 Port Data Register */
		{
			p1dat_set.reg = P1DAT;
			p1dat_set.bit.P14OUT = 1;
			P1DAT = p1dat_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25)
	/// Negate SPISS.
	if(spiChannel == 0)
	{
		/**
		 * @todo change PORT
		 *
		 * P00=PORT(Master)
		 */
		/// - P00 : High.
		/* P0DAT:P0 Port Data Register */
		{
			p0dat_set.reg = P0DAT;
			p0dat_set.bit.P00OUT = 1;
			P0DAT = p0dat_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_M01)
	else if(spiChannel == 1)
	{
		/**
		 * @todo change PORT
		 *
		 * P00=PORT(Master)
		 */
		/// - P00 : High.
		/* P0DAT:P0 Port Data Register */
		{
			p0dat_set.reg = P0DAT;
			p0dat_set.bit.P00OUT = 1;
			P0DAT = p0dat_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_W23)
	else if(spiChannel == 1)
	{
		/**
		 * @todo change PORT
		 *
		 * P36=PORT(Master)
		 */
		/// - P36 : High.
		/* P3DAT:P3 Port Data Register */
		{
			p3dat_set.reg = P3DAT;
			p3dat_set.bit.P36OUT = 1;
			P3DAT = p3dat_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36) || \
	defined(C17_MCUSEL_W03) || defined(C17_MCUSEL_W04) || \
	defined(C17_MCUSEL_W14) || defined(C17_MCUSEL_W16) || \
	defined(C17_MCUSEL_W18) || \
	defined(C17_MCUSEL_M40)
	else if(spiChannel == 1)
	{
		/**
		 * @todo change PORT
		 *
		 * P14=PORT(Master)
		 */
		/// - P14 : High.
		/* P1DAT:P1 Port Data Register */
		{
			p1dat_set.reg = P1DAT;
			p1dat_set.bit.P14OUT = 1;
			P1DAT = p1dat_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34) || \
	defined(C17_MCUSEL_589)
	else if(spiChannel == 1)
	{
		/**
		 * @todo change PORT
		 *
		 * P30=PORT(Master)
		 */
		/// - P30 : High.
		/* P3DAT:P3 Port Data Register */
		{
			p3dat_set.reg = P3DAT;
			p3dat_set.bit.P30OUT = 1;
			P3DAT = p3dat_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_M20) || defined(C17_MCUSEL_M21) || defined(C17_MCUSEL_M22) || defined(C17_MCUSEL_M23) || defined(C17_MCUSEL_M24) || defined(C17_MCUSEL_M25)
	else if(spiChannel == 1)
	{
		/**
		 * @todo change PORT
		 *
		 * P24=PORT(Master)
		 */
		/// - P24 : High.
		/* P2DAT:P2 Port Data Register */
		{
			p2dat_set.reg = P2DAT;
			p2dat_set.bit.P24OUT = 1;
			P2DAT = p2dat_set.reg;
		}
	}
#endif
#if defined(C17_MCUSEL_W16)
	else if(spiChannel == 2)
	{
		/**
		 * @todo change PORT
		 *
		 * P20=PORT(Master)
		 */
		/// - P04 : High.
		/* P2DAT:P2 Port Data Register */
		{
			p2dat_set.reg = P2DAT;
			p2dat_set.bit.P20OUT = 1;
			P2DAT = p2dat_set.reg;
		}
	}
#endif
}


/************************************************************************//**
 * @brief   SPIA transmit data write.
 *
 * @param   spiChannel    SPI CH
 * @param   spiData       Pointer of transmit data.
 * @param   spiSize       Transmit data size.
 *
 * @retval  none
 ***************************************************************************/
void c17sendSpiaData(unsigned short spiChannel, char *spiData, unsigned short spiSize)
{
	unsigned short i;

	// Send data.
	for (i = 0 ; i < spiSize ; i++) {
		/// Check transmit buffer empty.
		while (spinintf_table[spiChannel]->bit.TBEIF == 0) {
			/**
			 * Wait TBEIF Interrupt  for the send buff empty.
			 */
			/**
			 * @todo Add time-out processing as needed.
			 */
		}

		/// Send data.
		spintxd_table[spiChannel]->reg = *spiData++;
	}

	/// Wait until idol state.
	while (spinintf_table[spiChannel]->bit.BSY == 1) {
		/**
		 * Wait BSY Interrupt.
		 */
		/**
		 * @todo Add time-out processing as needed.
		 */
	}
}


/************************************************************************//**
 * @brief   SPIA receive data.
 *
 * @param   spiChannel    SPI CH
 * @param   spiData       Pointer of transmit data.
 * @param   spiSize       Transmit data size.
 *
 * @retval  none
 ***************************************************************************/
void c17rcvSpiaData(unsigned short spiChannel, char *spiData, unsigned short spiSize)
{
	unsigned short i;
	unsigned char dummy;

	/// RBFIF Clear.
	*spiData = spinrxd_table[spiChannel]->reg;

	/// Dummy Read data.
	dummy = 0;

	/// Receive data.
	for (i = 0 ; i < spiSize ; i++) {

		/// Set transmit dummy data(Master mode only).
		if (spinmod_table[spiChannel]->bit.MST == C17_SPIA_MODE_MASTER) {
			/// Check transmit buffer empty.
			while (spinintf_table[spiChannel]->bit.TBEIF == 0) {
				/**
				 * Wait TBEIF Interrupt  for the send buff empty.
				 */
				/**
				 * @todo Add time-out processing as needed.
				 */
			}
			/// Send data.
			spintxd_table[spiChannel]->reg = dummy;
		}

		/// Check receive buffer full.
		while (spinintf_table[spiChannel]->bit.RBFIF == 0) {
			/**
			 * Wait RBFIF Interrupt  for the receive buff full.
			 */
			/**
			 * @todo Add time-out processing as needed.
			 */
		}
		/// Read receive data.
		*spiData++	= spinrxd_table[spiChannel]->reg;
	}

	/// Wait until idol state.
	while (spinintf_table[spiChannel]->bit.BSY == 1) {
		/**
		 * Wait BSY Interrupt.
		 */
		/**
		 * @todo Add time-out processing as needed.
		 */
	}
}
#endif

