/************************************************************************//**
 *
 * @file    remc3_reg.h
 *
 * @brief   Definition of the Ir Remote Controller(REMC3) Register.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2016-2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 01 Nov. 2018    1.10
 *              - Change source code.
 *
 ******************************************************************************/
#ifndef REMC3_REG_H__
#define REMC3_REG_H__

#ifdef __cplusplus
extern "C" {
#endif

/// REMC3 Clock Control Register.
typedef union REMCLK_REG_TAG {
    unsigned short  reg;
    struct REMCLK_REG_BIT_TAG {
        /**
         * These bits select the clock source of the REMC3.
         * .CLKSRC[1:0]
         *  These bits select the clock source of the REMC3.
         * .CLKDIV[3:0]
         *  These bits select the division ratio of the REMC3 operating clock.
         * +-------------+-----------------------------------+
         * |             |           CLKSRC[1:0]             |
         * |             +--------+--------+--------+--------+
         * | CLKDIV[3:0] |  0x0   |   0x1  |  0x2   |  0x3   |
         * |             +--------+--------+--------+--------+
         * |             |  IOSC  |  OSC1  |  OSC3  | EXOSC  |
         * +-------------+--------+--------+--------+--------+
         * |     0xf     |1/32,768|1/1     |1/32,768|1/1     |
         * +-------------+--------+        +--------+        +
         * |     0xe     |1/16,384|        |1/16,384|        |
         * +-------------+--------+        +--------+        +
         * |     0xd     |1/8,192 |        |1/8,192 |        |
         * +-------------+--------+        +--------+        +
         * |     0xc     |1/4,096 |        |1/4,096 |        |
         * +-------------+--------+        +--------+        +
         * |     0xb     |1/2,048 |        |1/2,048 |        |
         * +-------------+--------+        +--------+        +
         * |     0xa     |1/1,024 |        |1/1,024 |        |
         * +-------------+--------+        +--------+        +
         * |     0x9     |1/512   |        |1/512   |        |
         * +-------------+--------+--------+--------+        +
         * |     0x8     |1/256   |1/256   |1/256   |        |
         * +-------------+--------+--------+--------+        +
         * |     0x7     |1/128   |1/128   |1/128   |        |
         * +-------------+--------+--------+--------+        +
         * |     0x6     |1/64    |1/64    |1/64    |        |
         * +-------------+--------+--------+--------+        +
         * |     0x5     |1/32    |1/32    |1/32    |        |
         * +-------------+--------+--------+--------+        +
         * |     0x4     |1/16    |1/16    |1/16    |        |
         * +-------------+--------+--------+--------+        +
         * |     0x3     |1/8     |1/8     |1/8     |        |
         * +-------------+--------+--------+--------+        +
         * |     0x2     |1/4     |1/4     |1/4     |        |
         * +-------------+--------+--------+--------+        +
         * |     0x1     |1/2     |1/2     |1/2     |        |
         * +-------------+--------+--------+--------+        +
         * |     0x0     |1/1     |1/1     |1/1     |        |
         * +-------------+--------+--------+--------+--------+
         */
        unsigned short      CLKSRC      : 2;
        unsigned short      reserved_1  : 2;    ///< Reserved.
        /**
         * These bits select the division ratio of the REMC3 operating clock.
         * .CLKSRC[1:0]
         *  These bits select the clock source of the REMC3.
         * .CLKDIV[3:0]
         *  These bits select the division ratio of the REMC3 operating clock.
         * +-------------+-----------------------------------+
         * |             |           CLKSRC[1:0]             |
         * |             +--------+--------+--------+--------+
         * | CLKDIV[3:0] |  0x0   |   0x1  |  0x2   |  0x3   |
         * |             +--------+--------+--------+--------+
         * |             |  IOSC  |  OSC1  |  OSC3  | EXOSC  |
         * +-------------+--------+--------+--------+--------+
         * |     0xf     |1/32,768|1/1     |1/32,768|1/1     |
         * +-------------+--------+        +--------+        +
         * |     0xe     |1/16,384|        |1/16,384|        |
         * +-------------+--------+        +--------+        +
         * |     0xd     |1/8,192 |        |1/8,192 |        |
         * +-------------+--------+        +--------+        +
         * |     0xc     |1/4,096 |        |1/4,096 |        |
         * +-------------+--------+        +--------+        +
         * |     0xb     |1/2,048 |        |1/2,048 |        |
         * +-------------+--------+        +--------+        +
         * |     0xa     |1/1,024 |        |1/1,024 |        |
         * +-------------+--------+        +--------+        +
         * |     0x9     |1/512   |        |1/512   |        |
         * +-------------+--------+--------+--------+        +
         * |     0x8     |1/256   |1/256   |1/256   |        |
         * +-------------+--------+--------+--------+        +
         * |     0x7     |1/128   |1/128   |1/128   |        |
         * +-------------+--------+--------+--------+        +
         * |     0x6     |1/64    |1/64    |1/64    |        |
         * +-------------+--------+--------+--------+        +
         * |     0x5     |1/32    |1/32    |1/32    |        |
         * +-------------+--------+--------+--------+        +
         * |     0x4     |1/16    |1/16    |1/16    |        |
         * +-------------+--------+--------+--------+        +
         * |     0x3     |1/8     |1/8     |1/8     |        |
         * +-------------+--------+--------+--------+        +
         * |     0x2     |1/4     |1/4     |1/4     |        |
         * +-------------+--------+--------+--------+        +
         * |     0x1     |1/2     |1/2     |1/2     |        |
         * +-------------+--------+--------+--------+        +
         * |     0x0     |1/1     |1/1     |1/1     |        |
         * +-------------+--------+--------+--------+--------+
         */
        unsigned short      CLKDIV      : 4;
        /**
         * This bit sets whether the REMC3 operating clock is supplied in DEBUG mode or not.
         * .DBRUN
         *  This bit sets whether the REMC3 operating clock is supplied in DEBUG mode or not.
         *    1 (R/W): Clock supplied in DEBUG mode
         *    0 (R/W): No clock supplied in DEBUG mode
         */
        unsigned short      DBRUN       : 1;
        unsigned short      reserved_2  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} REMCLK_REG;

/// remc Data Bit Counter Control Register.
typedef union REMDBCTL_REG_TAG {
    unsigned short  reg;
    struct REMDBCTL_REG_BIT_TAG {
        /**
         * This bit enables the REMC3 operations.
         * .MODEN
         *  This bit enables the REMC3 operations.
         *    1 (R/W): Enable REMC3 operations (The operating clock is supplied.)
         *    0 (R/W): Disable REMC3 operations (The operating clock is stopped.)
         */
        unsigned short      MODEN       : 1;
        /**
         * This bit issues software reset to the REMC3.
         * .REMCRST
         *  This bit issues software reset to the REMC3.
         *    1 (W): Issue software reset
         *    0 (W): Ineffective
         *    1 (R): Software reset is executing.
         *    0 (R): Software reset has finished. (During normal operation)
         */
        unsigned short      REMCRST     : 1;
        /**
         * This bit selects the operation mode of the 16-bit counter for data signal generation.
         * .TRMD
         *  This bit selects the operation mode of the 16-bit counter for data signal generation.
         *    1 (R/W): One-shot mode
         *    0 (R/W): Repeat mode
         */
        unsigned short      TRMD        : 1;
        /**
         * This bit enables or disables the compare buffers.
         * .BUFEN
         *  This bit enables or disables the compare buffers.
         *    1 (R/W): Enable
         *    0 (R/W): Disable
         */
        unsigned short      BUFEN       : 1;
        /**
         * This bit inverts the REMO output signal.
         * .REMOINV
         *  This bit inverts the REMO output signal.
         *    1 (R/W): Inverted
         *    0 (R/W): Non-inverted
         */
        unsigned short      REMOINV     : 1;
        unsigned short      reserved_1  : 3;    ///< Reserved.
        /**
         * This bit starts/stops counting by the internal counters.
         * .PRUN
         *  This bit starts/stops counting by the internal counters (16-bit counter for data signal generation and 8-bit counter for carrier generation).
         *    1 (W): Start counting
         *    0 (W): Stop counting
         *    1 (R): Counting
         *    0 (R): Idle
         */
        unsigned short      PRUN        : 1;
        /**
         * This bit resets the internal counters.
         * .PRESET
         *  This bit resets the internal counters (16-bit counter for data signal generation and 8-bit counter for carriergeneration).
         *    1 (W): Reset
         *    0 (W): Ineffective
         *    1 (R): Resetting in progress
         *    0 (R): Resetting finished or normal operation
         */
        unsigned short      PRESET      : 1;
        unsigned short      reserved_2  : 6;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} REMDBCTL_REG;

/// REMC3 Data Bit Counter Register.
typedef union REMDBCNT_REG_TAG {
    unsigned short  reg;
    struct REMDBCNT_REG_BIT_TAG {
        /**
         * REMC3 Data Bit Counter Register.
         * .DBCNT[15:0]
         *  The current value of the 16-bit counter for data signal generation can be read out through these bits.
         */
        unsigned short      DBCNT       :16;
    } bit;
    unsigned char   byte_reg[2];
} REMDBCNT_REG;

/// REMC3 Data Bit Active Pulse Length Register.
typedef union REMAPLEN_REG_TAG {
    unsigned short  reg;
    struct REMAPLEN_REG_BIT_TAG {
        /**
         * REMC3 Data Bit Active Pulse Length Register.
         * .APLEN[15:0]
         *  These bits set the active pulse length of the data signal.
         */
        unsigned short      APLEN       :16;
    } bit;
    unsigned char   byte_reg[2];
} REMAPLEN_REG;

/// REMC3 Data Bit Length Register.
typedef union REMDBLEN_REG_TAG {
    unsigned short  reg;
    struct REMDBLEN_REG_BIT_TAG {
        /**
         * REMC3 Data Bit Length Register.
         * .DBLEN[15:0]
         *  These bits set the data length of the data signal (length of one cycle).
         */
        unsigned short      DBLEN       :16;
    } bit;
    unsigned char   byte_reg[2];
} REMDBLEN_REG;

/// REMC3 Status and Interrupt Flag Register.
typedef union REMINTF_REG_TAG {
    unsigned short  reg;
    struct REMINTF_REG_BIT_TAG {
        /**
         * This bit indicate the REMC3 Compare AP interrupt cause occurrence status.
         * .APIF
         *  This bit indicate the REMC3 interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         * .APIF bit: Compare AP interrupt
         */
        unsigned short      APIF        : 1;
        /**
         * This bit indicate the REMC3 Compare DB interrupt cause occurrence status.
         * .DBIF
         *  This bit indicate the REMC3 interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         * .DBIF bit: Compare DB interrupt
         */
        unsigned short      DBIF        : 1;
        unsigned short      reserved_1  : 6;    ///< Reserved.
        /**
         * This bit indicates whether a compare match occurs by the REMAPLEN buffer or not.
         * .APLENBSY
         *  This bit indicates whether the value written to the REMAPLEN.APLEN[15:0] bits is transferred to
         *  the REMAPLEN buffer or not.
         *    1 (R): Transfer to the REMAPLEN buffer has not completed.
         *    0 (R): Transfer to the REMAPLEN buffer has completed.
         */
        unsigned short      APLENBSY    : 1;
        /**
         * This bit indicates whether a compare match occurs by the REMDBLEN buffer or not.
         * .DBLENBSY
         *  This bit indicates whether the value written to the REMDBLEN.DBLEN[15:0] bits is transferred to
         *  the REMDBLEN buffer or not.
         *    1 (R): Transfer to the REMDBLEN buffer has not completed.
         *    0 (R): Transfer to the REMDBLEN buffer has completed.
         */
        unsigned short      DBLENBSY    : 1;
        /**
         * This bit indicates whether the 16-bit counter for data signal generation is running or not.
         * .DBCNTRUN
         *  This bit indicates whether the 16-bit counter for data signal generation is running or not.
         *    1 (R): Running (Counting)
         *    0 (R): Idle
         */
        unsigned short      DBCNTRUN    : 1;
        unsigned short      reserved_2  : 5;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} REMINTF_REG;

/// REMC3 Interrupt Enable Register.
typedef union REMINTE_REG_TAG {
    unsigned short  reg;
    struct REMINTE_REG_BIT_TAG {
        /**
         * This bit enable REMC3 Compare AP interrupt.
         * .APIE
         *  This bit enable REMC3 interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         * .APIE bit: Compare AP interrupt
         */
        unsigned short      APIE        : 1;
        /**
         * This bit enable REMC3 Compare DB interrupt.
         * .DBIE
         *  This bit enable REMC3 interrupts.
         *    1 (R/W): Enable interrupts
         *    0 (R/W): Disable interrupts
         * .DBIE bit: Compare DB interrupt
         */
        unsigned short      DBIE        : 1;
        unsigned short      reserved_1  :14;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} REMINTE_REG;

/// REMC3 Carrier Waveform Register.
typedef union REMCARR_REG_TAG {
    unsigned short  reg;
    struct REMCARR_REG_BIT_TAG {
        /**
         * These bits set the carrier signal cycle.
         * .CRPER[7:0]
         *  These bits set the carrier signal cycle.
         */
        unsigned short      CRPER       : 8;
        /**
         * These bits set the high level period of the carrier signal.
         * .CRDTY[7:0]
         *  These bits set the high level period of the carrier signal.
         */
        unsigned short      CRDTY       : 8;
    } bit;
    unsigned char   byte_reg[2];
} REMCARR_REG;

/// REMC3 Carrier Modulation Control Register.
typedef union REMCCTL_REG_TAG {
    unsigned short  reg;
    struct REMCCTL_REG_BIT_TAG {
        /**
         * REMC3 Carrier Modulation Control Register.
         * .CARREN
         *  This bit enables carrier modulation.
         *    1 (R/W): Enable carrier modulation
         *    0 (R/W): Disable carrier modulation (output data signal only)
         */
        unsigned short      CARREN      : 1;
        unsigned short      reserved_1  : 7;    ///< Reserved.
        /**
         * This bit inverts the REMO output polarity.
         * .OUTINVEN
         *  This bit inverts the REMO output polarity.
         *    1 (R/W): Inverted
         *    0 (R/W): Non-inverted
         */
        unsigned short      OUTINVEN    : 1;
        unsigned short      reserved_2  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} REMCCTL_REG;

#ifdef __cplusplus
}
#endif

#endif // REMC3_REG_H__
